@@ -323,6 +323,88 @@ exit_fail:
     return NULL;
 }
 
+/* Convert disks array to list of tuples. */
+static PyObject *
+disks_to_list(struct sanlk_disk *disks, uint32_t disks_count)
+{
+    PyObject *result = NULL;
+    PyObject *disk = NULL;
+
+    result = PyList_New(disks_count);
+    if (result == NULL)
+        return NULL;
+
+    for (uint32_t i = 0; i < disks_count; i++) {
+        disk = Py_BuildValue(
+            "(s,K)",
+            disks[i].path,
+            disks[i].offset);
+        if (disk == NULL)
+            goto exit_fail;
+
+        /* Steals reference to disk. */
+        if (PyList_SetItem(result, i, disk) != 0)
+            goto exit_fail;
+
+        disk = NULL;
+    }
+
+    return result;
+
+exit_fail:
+    Py_XDECREF(result);
+    Py_XDECREF(disk);
+
+    return NULL;
+}
+
+/* Convert resources array returned from sanlock_inquire() to list of resource
+ * dicts. */
+static PyObject *
+resources_to_list(struct sanlk_resource **res, int res_count)
+{
+    PyObject *result = NULL;
+    PyObject *info = NULL;
+    PyObject *disks = NULL;
+
+    if ((result = PyList_New(res_count)) == NULL)
+        return NULL;
+
+    for (int i = 0; i < res_count; i++) {
+        disks = disks_to_list(res[i]->disks, res[i]->num_disks);
+        if (disks == NULL)
+            goto exit_fail;
+
+        /* Steals reference to disks. */
+        info = Py_BuildValue(
+            "{s:y,s:y,s:k,s:K,s:N}",
+            "lockspace", res[i]->lockspace_name,
+            "resource", res[i]->name,
+            "flags", res[i]->flags,
+            "version", res[i]->lver,
+            "disks", disks);
+        if (info  == NULL)
+            goto exit_fail;
+
+        disks = NULL;
+
+        /* Steals reference to info. */
+        if (PyList_SetItem(result, i, info) != 0)
+            goto exit_fail;
+
+        info = NULL;
+    }
+
+    return result;
+
+exit_fail:
+    Py_XDECREF(result);
+    Py_XDECREF(info);
+    Py_XDECREF(disks);
+
+    return NULL;
+}
+
 /* register */
 PyDoc_STRVAR(pydoc_register, "\
 register() -> int\n\
@@ -420,107 +502,6 @@ convert_to_pybytes(PyObject* arg, void *addr)
     return 0;
 }
 
-/* init_lockspace */
-PyDoc_STRVAR(pydoc_init_lockspace, "\
-init_lockspace(lockspace, path, offset=0, max_hosts=0, num_hosts=0, \
-use_aio=True)\n\
-*DEPRECATED* use write_lockspace instead.\n\
-Initialize a device to be used as sanlock lockspace.");
-
-static PyObject *
-py_init_lockspace(PyObject *self __unused, PyObject *args, PyObject *keywds)
-{
-    int rv = -1, max_hosts = 0, num_hosts = 0, use_aio = 1;
-    PyObject *lockspace = NULL;
-    PyObject *path = NULL;
-    struct sanlk_lockspace ls = {0};
-
-    static char *kwlist[] = {"lockspace", "path", "offset",
-                                "max_hosts", "num_hosts", "use_aio", NULL};
-
-    /* parse python tuple */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O&O&|kiii", kwlist,
-        convert_to_pybytes, &lockspace, pypath_converter, &path, &ls.host_id_disk.offset,
-        &max_hosts, &num_hosts, &use_aio)) {
-        goto finally;
-    }
-
-    /* prepare sanlock names */
-    strncpy(ls.name, PyBytes_AsString(lockspace), SANLK_NAME_LEN);
-    strncpy(ls.host_id_disk.path, PyBytes_AsString(path), SANLK_PATH_LEN - 1);
-
-    /* init sanlock lockspace (gil disabled) */
-    Py_BEGIN_ALLOW_THREADS
-    rv = sanlock_direct_init(&ls, NULL, max_hosts, num_hosts, use_aio);
-    Py_END_ALLOW_THREADS
-
-    if (rv != 0) {
-        set_sanlock_error(rv, "Sanlock lockspace init failure");
-        goto finally;
-    }
-
-finally:
-    Py_XDECREF(lockspace);
-    Py_XDECREF(path);
-    if (rv != 0)
-        return NULL;
-    Py_RETURN_NONE;
-}
-
-/* init_resource */
-PyDoc_STRVAR(pydoc_init_resource, "\
-init_resource(lockspace, resource, disks, max_hosts=0, num_hosts=0, \
-use_aio=True)\n\
-*DEPRECATED* use write_resource instead.\n\
-Initialize a device to be used as sanlock resource.\n\
-The disks must be in the format: [(path, offset), ... ]");
-
-static PyObject *
-py_init_resource(PyObject *self __unused, PyObject *args, PyObject *keywds)
-{
-    int rv = -1, max_hosts = 0, num_hosts = 0, use_aio = 1;
-    PyObject *lockspace = NULL, *resource = NULL;
-    struct sanlk_resource *res = NULL;
-    PyObject *disks;
-
-    static char *kwlist[] = {"lockspace", "resource", "disks", "max_hosts",
-                                "num_hosts", "use_aio", NULL};
-
-    /* parse python tuple */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O&O&O!|iii",
-        kwlist, convert_to_pybytes, &lockspace, convert_to_pybytes, &resource,
-        &PyList_Type, &disks, &max_hosts, &num_hosts, &use_aio)) {
-        goto finally;
-    }
-
-    /* parse and check sanlock resource */
-    if (parse_disks(disks, &res) < 0) {
-        goto finally;
-    }
-
-    /* prepare sanlock names */
-    strncpy(res->lockspace_name, PyBytes_AsString(lockspace), SANLK_NAME_LEN);
-    strncpy(res->name, PyBytes_AsString(resource), SANLK_NAME_LEN);
-
-    /* init sanlock resource (gil disabled) */
-    Py_BEGIN_ALLOW_THREADS
-    rv = sanlock_direct_init(NULL, res, max_hosts, num_hosts, use_aio);
-    Py_END_ALLOW_THREADS
-
-    if (rv != 0) {
-        set_sanlock_error(rv, "Sanlock resource init failure");
-        goto finally;
-    }
-
-finally:
-    Py_XDECREF(lockspace);
-    Py_XDECREF(resource);
-    free(res);
-    if (rv != 0)
-        return NULL;
-    Py_RETURN_NONE;
-}
-
 /* write_lockspace */
 PyDoc_STRVAR(pydoc_write_lockspace, "\
 write_lockspace(lockspace, path, offset=0, max_hosts=0, iotimeout=0, \
@@ -790,7 +771,7 @@ finally:
 PyDoc_STRVAR(pydoc_add_lockspace, "\
 add_lockspace(lockspace, host_id, path, offset=0, iotimeout=0, wait=True)\n\
 Add a lockspace, acquiring a host_id in it. If wait is False the function\n\
-will return immediatly and the status can be checked using inq_lockspace.\n\
+will return immediately and the status can be checked using inq_lockspace.\n\
 The iotimeout option configures the io timeout for the specific lockspace,\n\
 overriding the default value (see the sanlock daemon parameter -o).");
 
@@ -1082,28 +1063,31 @@ finally:
 /* acquire */
 PyDoc_STRVAR(pydoc_acquire, "\
 acquire(lockspace, resource, disks \
-[, slkfd=fd, pid=owner, shared=False, version=None])\n\
+[, slkfd=fd, pid=owner, shared=False, version=None, lvb=False])\n\
 Acquire a resource lease for the current process (using the slkfd argument\n\
-to specify the sanlock file descriptor) or for an other process (using the\n\
+to specify the sanlock file descriptor) or for another process (using the\n\
 pid argument). If shared is True the resource will be acquired in the shared\n\
 mode. The version is the version of the lease that must be acquired or fail.\n\
-The disks must be in the format: [(path, offset), ... ]\n");
+The disks must be in the format: [(path, offset), ... ]\n\
+If lvb is True the resource will be acquired with the LVB flag enabled\n\
+to allow access to LVB data.\n");
 
 static PyObject *
 py_acquire(PyObject *self __unused, PyObject *args, PyObject *keywds)
 {
-    int rv = -1, sanlockfd = -1, pid = -1, shared = 0;
+    int rv = -1, sanlockfd = -1, pid = -1, shared = 0, lvb = 0;
+    uint32_t flags = 0;
     PyObject *lockspace = NULL, *resource = NULL;
     struct sanlk_resource *res = NULL;
     PyObject *disks, *version = Py_None;
 
     static char *kwlist[] = {"lockspace", "resource", "disks", "slkfd",
-                                "pid", "shared", "version", NULL};
+                                "pid", "shared", "lvb", "version", NULL};
 
     /* parse python tuple */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O&O&O!|iiiO", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O&O&O!|iiiiO", kwlist,
         convert_to_pybytes, &lockspace, convert_to_pybytes, &resource,
-        &PyList_Type, &disks, &sanlockfd, &pid, &shared, &version)) {
+        &PyList_Type, &disks, &sanlockfd, &pid, &shared, &lvb, &version)) {
         goto finally;
     }
 
@@ -1127,6 +1111,10 @@ py_acquire(PyObject *self __unused, PyObject *args, PyObject *keywds)
         res->flags |= SANLK_RES_SHARED;
     }
 
+    if (lvb) {
+        flags |= SANLK_ACQUIRE_LVB;
+    }
+
     /* prepare the resource version */
     if (version != Py_None) {
         res->flags |= SANLK_RES_LVER;
@@ -1139,7 +1127,7 @@ py_acquire(PyObject *self __unused, PyObject *args, PyObject *keywds)
 
     /* acquire sanlock resource (gil disabled) */
     Py_BEGIN_ALLOW_THREADS
-    rv = sanlock_acquire(sanlockfd, pid, 0, 1, &res, 0);
+    rv = sanlock_acquire(sanlockfd, pid, flags, 1, &res, 0);
     Py_END_ALLOW_THREADS
 
     if (rv != 0) {
@@ -1156,6 +1144,89 @@ finally:
     Py_RETURN_NONE;
 }
 
+/* inquire */
+PyDoc_STRVAR(pydoc_inquire, "\
+inquire(slkfd=-1, pid=-1)\n\
+Return list of resources held by current process (using the slkfd \n\
+argument to specify the sanlock file descriptor) or for another \n\
+process (using the pid argument).\n\
+\n\
+Does not access storage. To learn about resource state on storage,\n\
+use sanlock.read_resource() and sanlock.read_resource_owners().\n\
+\n\
+Arguments\n\
+  slkfd (int):  The file descriptor returned from sanlock.register().\n\
+  pid (int):    The program pid to query.\n\
+\n\
+Returns\n\
+    List of resource dicts with the following keys:\n\
+      lockspace (bytes):    lockspace name\n\
+      resource (bytes):     resource name\n\
+      flags (int):          resource flags (sanlock.RES_*)\n\
+      version (int):        resource version\n\
+      disks (list):         list of disk tuples (path, offset)\n\
+");
+
+static PyObject *
+py_inquire(PyObject *self __unused, PyObject *args, PyObject *keywds)
+{
+    int sanlockfd = -1;
+    int pid = -1;
+    char *kwlist[] = {"slkfd", "pid", NULL};
+    int rv = -1;
+
+    /* sanlock_inquire() return values. */
+    int res_count = 0;
+    char *res_state = NULL;
+
+    /* Array of resoruces parsed from res_state. */
+    struct sanlk_resource **res_arr = NULL;
+
+    /* List of resource dicts. */
+    PyObject *result = NULL;
+
+    if (!PyArg_ParseTupleAndKeywords(
+            args, keywds, "|ii", kwlist, &sanlockfd, &pid)) {
+        return NULL;
+    }
+
+    /* Check if any of the slkfd or pid parameters was given. */
+    if (sanlockfd == -1 && pid == -1) {
+        set_sanlock_error(-EINVAL, "Invalid slkfd and pid values");
+        return NULL;
+    }
+
+    /* Inquire sanlock (gil disabled) */
+    Py_BEGIN_ALLOW_THREADS
+    rv = sanlock_inquire(sanlockfd, pid, 0, &res_count, &res_state);
+    Py_END_ALLOW_THREADS
+
+    if (rv != 0) {
+        set_sanlock_error(rv, "Inquire error");
+        return NULL;
+    }
+
+    if (res_count > 0) {
+        rv = sanlock_state_to_args(res_state, &res_count, &res_arr);
+        if (rv != 0) {
+            /* TODO: Include res_state in the error. */
+            set_sanlock_error(rv, "Error parsing inquire state string");
+            goto finally;
+        }
+    }
+
+    result = resources_to_list(res_arr, res_count);
+
+finally:
+    free(res_state);
+
+    for (int i = 0; i < res_count; i++)
+        free(res_arr[i]);
+    free(res_arr);
+
+    return result;
+}
+
 /* release */
 PyDoc_STRVAR(pydoc_release, "\
 release(lockspace, resource, disks [, slkfd=fd, pid=owner])\n\
@@ -1688,14 +1759,142 @@ finally:
     Py_RETURN_NONE;
 }
 
+/* set_lvb */
+PyDoc_STRVAR(pydoc_set_lvb, "\
+set_lvb(lockspace, resource, disks, data)\n\
+Set Lock Value Block for a given resource\n\
+\n\
+Arguments\n\
+  lockspace         lockspace name (str)\n\
+  resource          resource name (int)\n\
+  disks             path and offset (tuple)\n\
+  data              data to write (bytes)\n\
+\n\
+Notes\n\
+  The resource must be acquired with lvb=true.\n\
+  The size of data is limited by the sector size (512/4K).\n\
+  The new data is visible after the resource is released.\n\
+");
+
+static PyObject *
+py_set_lvb(PyObject *self __unused, PyObject *args, PyObject *keywds)
+{
+    uint32_t flags = 0;
+    int rv = -1;
+    struct sanlk_resource *res = NULL;
+    PyObject *lockspace = NULL, *resource = NULL, *data = NULL;
+    PyObject *disks;
+
+    static char *kwlist[] = {"lockspace", "resource", "disks", "data", NULL};
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O&O&O!O&", kwlist,
+        convert_to_pybytes, &lockspace, convert_to_pybytes, &resource,
+        &PyList_Type, &disks, convert_to_pybytes, &data)) {
+        goto finally;
+    }
+
+    if (parse_disks(disks, &res) < 0) {
+        goto finally;
+    }
+
+    strncpy(res->lockspace_name, PyBytes_AsString(lockspace), SANLK_NAME_LEN);
+    strncpy(res->name, PyBytes_AsString(resource), SANLK_NAME_LEN);
+
+    Py_BEGIN_ALLOW_THREADS
+    rv = sanlock_set_lvb(flags, res, PyBytes_AS_STRING(data), PyBytes_GET_SIZE(data));
+    Py_END_ALLOW_THREADS
+
+    if (rv < 0) {
+        set_sanlock_error(rv, "Unable to set lvb");
+        goto finally;
+    }
+
+finally:
+    Py_XDECREF(lockspace);
+    Py_XDECREF(resource);
+    free(res);
+    if (rv < 0)
+        return NULL;
+    Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(pydoc_get_lvb, "\
+get_lvb(lockspace, resource, disks, size) -> bytes\n\
+Read Lock Value Block for a given resource\n\
+\n\
+Arguments\n\
+  lockspace         lockspace name (str)\n\
+  resource          resource name (int)\n\
+  disks             path and offset (tuple)\n\
+  size              amount of data to read (int)\n\
+\n\
+Returns\n\
+  data              data written with set_lvb\n\
+\n\
+Notes\n\
+  The resource must be acquired with lvb=True.\n\
+  size has to be in range 0 < n <= 4096.\n\
+");
+
+static PyObject *
+py_get_lvb(PyObject *self __unused, PyObject *args, PyObject *keywds)
+{
+    uint32_t flags = 0;
+    uint32_t lvb_len = 0;
+    int rv = -1;
+    struct sanlk_resource *res = NULL;
+    PyObject *lockspace = NULL, *resource = NULL;
+    PyObject *disks = NULL;
+    PyObject *result = NULL;
+
+    static char *kwlist[] = {"lockspace", "resource", "disks", "size", NULL};
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O&O&O!I", kwlist,
+        convert_to_pybytes, &lockspace, convert_to_pybytes, &resource,
+        &PyList_Type, &disks, &lvb_len)) {
+        goto finally;
+    }
+
+    if (lvb_len < 1 || lvb_len > 4096) {
+        PyErr_Format(PyExc_ValueError, "Invalid size %d, must be in range: 0 < size <= 4096", lvb_len);
+        goto finally;
+    }
+
+    if (parse_disks(disks, &res) < 0) {
+        goto finally;
+    }
+
+    strncpy(res->lockspace_name, PyBytes_AsString(lockspace), SANLK_NAME_LEN);
+    strncpy(res->name, PyBytes_AsString(resource), SANLK_NAME_LEN);
+
+    result = PyBytes_FromStringAndSize(NULL, lvb_len);
+    if (result == NULL) {
+        goto finally;
+    }
+
+    memset(PyBytes_AS_STRING(result), 0, lvb_len);
+
+    Py_BEGIN_ALLOW_THREADS
+    rv = sanlock_get_lvb(flags, res, PyBytes_AS_STRING(result), lvb_len);
+    Py_END_ALLOW_THREADS
+
+    if (rv < 0) {
+        set_sanlock_error(rv, "Unable to get lvb");
+        goto finally;
+    }
+
+finally:
+    Py_XDECREF(lockspace);
+    Py_XDECREF(resource);
+    free(res);
+    if (rv < 0)
+        Py_CLEAR(result);
+
+    return result;
+}
+
 static PyMethodDef
 sanlock_methods[] = {
     {"register", py_register, METH_NOARGS, pydoc_register},
     {"get_alignment", py_get_alignment, METH_VARARGS, pydoc_get_alignment},
-    {"init_lockspace", (PyCFunction) py_init_lockspace,
-                        METH_VARARGS|METH_KEYWORDS, pydoc_init_lockspace},
-    {"init_resource", (PyCFunction) py_init_resource,
-                        METH_VARARGS|METH_KEYWORDS, pydoc_init_resource},
     {"write_lockspace", (PyCFunction) py_write_lockspace,
                         METH_VARARGS|METH_KEYWORDS, pydoc_write_lockspace},
     {"write_resource", (PyCFunction) py_write_resource,
@@ -1718,6 +1917,8 @@ sanlock_methods[] = {
                 METH_VARARGS|METH_KEYWORDS, pydoc_read_resource_owners},
     {"acquire", (PyCFunction) py_acquire,
                 METH_VARARGS|METH_KEYWORDS, pydoc_acquire},
+    {"inquire", (PyCFunction) py_inquire,
+                METH_VARARGS|METH_KEYWORDS, pydoc_inquire},
     {"release", (PyCFunction) py_release,
                 METH_VARARGS|METH_KEYWORDS, pydoc_release},
     {"request", (PyCFunction) py_request,
@@ -1729,6 +1930,11 @@ sanlock_methods[] = {
     {"end_event", (PyCFunction) py_end_event, METH_VARARGS, pydoc_end_event},
     {"set_event", (PyCFunction) py_set_event,
                 METH_VARARGS|METH_KEYWORDS, pydoc_set_event},
+    {"set_lvb", (PyCFunction) py_set_lvb,
+         METH_VARARGS|METH_KEYWORDS, pydoc_set_lvb},
+    {"get_lvb", (PyCFunction) py_get_lvb,
+         METH_VARARGS|METH_KEYWORDS, pydoc_get_lvb},
+
     {NULL, NULL, 0, NULL}
 };
 
@@ -1811,6 +2017,12 @@ module_init(PyObject* m)
     if (PyModule_AddIntConstant(m, "SETEV_ALL_HOSTS", SANLK_SETEV_ALL_HOSTS))
         return -1;
 
+    /* sanlock_inquire() result resource flags */
+    if (PyModule_AddIntConstant(m, "RES_LVER", SANLK_RES_LVER))
+        return -1;
+    if (PyModule_AddIntConstant(m, "RES_SHARED", SANLK_RES_SHARED))
+        return -1;
+
     /* Tuples with supported sector size and alignment values */
 
     PyObject *sector = Py_BuildValue("ii", SECTOR_SIZE_512, SECTOR_SIZE_4K);
