@@ -180,6 +180,17 @@ static void status_host(struct sanlk_state *st, char *str, int debug)
 		print_debug(str, st->str_len);
 }
 
+static void status_worker(struct sanlk_state *st, char *str, int debug)
+{
+	if (!debug)
+		return;
+
+	printf("worker %u\n", st->data32);
+
+	if (st->str_len)
+		print_debug(str, st->str_len);
+}
+
 static void print_st(struct sanlk_state *st, char *str, char *bin, int debug)
 {
 	switch (st->type) {
@@ -195,6 +206,9 @@ static void print_st(struct sanlk_state *st, char *str, char *bin, int debug)
 	case SANLK_STATE_RESOURCE:
 		status_resource(st, str, bin, debug);
 		break;
+	case SANLK_STATE_WORKER:
+		status_worker(st, str, debug);
+		break;
 	}
 }
 
