@@ -21,6 +21,7 @@
 #include <syslog.h>
 #include <sys/types.h>
 #include <sys/time.h>
+#include <sys/mman.h>
 
 #include "sanlock_internal.h"
 #include "sanlock.h"
@@ -340,6 +341,8 @@ int delta_lease_acquire(struct task *task,
 	uint32_t checksum;
 	int other_io_timeout, other_host_dead_seconds, other_id_renewal_seconds;
 	int i, error, rv, delay, delta_large_delay;
+	int is_free, is_same;
+	int no_delay = 0;
 
 	log_space(sp, "delta_acquire begin %.48s:%llu",
 		  sp->space_name, (unsigned long long)host_id);
@@ -352,24 +355,58 @@ int delta_lease_acquire(struct task *task,
 	}
 
 	other_io_timeout = leader.io_timeout;
-
-	if (!other_io_timeout) {
-		log_erros(sp, "delta_acquire use own io_timeout %d", sp->io_timeout);
+	if (!other_io_timeout)
 		other_io_timeout = sp->io_timeout;
-	} else if (other_io_timeout != sp->io_timeout) {
-		log_erros(sp, "delta_acquire other_io_timeout %u our %u",
-			  leader.io_timeout, sp->io_timeout);
-	}
 
-	if (leader.timestamp == LEASE_FREE)
-		goto write_new;
+	/*
+	 * If the delta lease is free, and the prev owner matches our host
+	 * name, then reacquire with no delay.
+	 *
+	 * If the delta lease is not free, and the prev owner matches our host
+	 * name, and our host name is from product_uuid, then reacquire with no
+	 * delay.  Assumption here is that the delta lease is not free because
+	 * this host did not release it cleanly last time.  The non-free lease
+	 * generally indicates it's being used by someone, and we should
+	 * monitor it for a renewal period for updates.  But, we skip this
+	 * monitoring given the certainty that comes from the owner being
+	 * the product_uuid, and the assumption that our product_uuid will not
+	 * be used by another host.
+	 *
+	 * If the prev owner was not our name, we delay for a rewnewal period
+	 * to monitor for current use from another host.
+	 *
+	 * If the lease is not free, we delay and monitor for a renewal period
+	 * to monitor (except when owner is our product_uuid per above.)
+	 *
+	 * If the lease is not free, and the owner does not match our host
+	 * name, then use a long delay to monitor for other hosts using it,
+	 * or to ensure a prev host using this lease is dead.
+	 */
+
+	is_free = (leader.timestamp == LEASE_FREE);
+	is_same = !memcmp(leader.resource_name, our_host_name, NAME_ID_SIZE);
+
+	if (!is_same)
+		log_debug("delta_acquire new owner %.48s old owner %.48s", our_host_name, leader.resource_name);
+
+	if (is_same || is_free) {
+		if (is_same && is_free)
+			no_delay = 1;
+		if (is_same && our_host_name_matches_product_uuid)
+			no_delay = 1;
+
+		log_space(sp, "delta_acquire %s owner, %s free, %s our_product_uuid, %s delay, other_io_timeout %d",
+			  is_same ? "same" : "new",
+			  is_free ? "is" : "not",
+			  our_host_name_matches_product_uuid ? "is" : "not",
+			  no_delay ? "no" : "short",
+			  other_io_timeout);
 
-	if (!strncmp(leader.resource_name, our_host_name, NAME_ID_SIZE)) {
-		log_space(sp, "delta_acquire fast reacquire");
 		goto write_new;
 	}
 
-	/* we need to ensure that a host_id cannot be acquired and released
+	/*
+	 * we need to ensure that a host_id cannot be acquired and released
 	 * sooner than host_dead_seconds because the change in host_id
 	 * ownership affects the host_id "liveness" determination used by paxos
 	 * leases, and the ownership of paxos leases cannot change until after
@@ -385,18 +422,15 @@ int delta_lease_acquire(struct task *task,
 	 * paxos leases cannot change ownership until a min of
 	 * host_dead_seconds to ensure the watchdog has fired.  So, the timeout
 	 * we use here must be the max of the delta delay (D+6d) and
-	 * host_dead_seconds */
-
-	/*
-	 * delay = task->host_dead_seconds;
-	 * delta_large_delay = task->id_renewal_seconds + (6 * task->io_timeout_seconds);
-	 * if (delta_large_delay > delay)
-	 * 	delay = delta_large_delay;
+	 * host_dead_seconds.
 	 */
 
 	other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
 	other_id_renewal_seconds = calc_id_renewal_seconds(other_io_timeout);
 
+	log_space(sp, "delta_acquire other_host %.48s, timestamp %llu, other_io_timeout %d",
+		  leader.resource_name, (unsigned long long)leader.timestamp, other_io_timeout);
+
 	delay = other_host_dead_seconds;
 	delta_large_delay = other_id_renewal_seconds + (6 * other_io_timeout);
 	if (delta_large_delay > delay)
@@ -444,11 +478,12 @@ int delta_lease_acquire(struct task *task,
 
  write_new:
 	new_ts = monotime();
+	leader.version = DELTA_DISK_VERSION_MAJOR | DELTA_DISK_VERSION_MINOR;
 	leader.timestamp = new_ts;
 	leader.io_timeout = (sp->io_timeout & 0x00FF);
 	leader.owner_id = host_id;
 	leader.owner_generation++;
-	snprintf(leader.resource_name, NAME_ID_SIZE, "%s", our_host_name);
+	memcpy(leader.resource_name, our_host_name, NAME_ID_SIZE);
 	leader.checksum = 0; /* set below */
 
 	log_space(sp, "delta_acquire write %llu %llu %llu %.48s",
@@ -475,6 +510,11 @@ int delta_lease_acquire(struct task *task,
 
 	memcpy(&leader1, &leader, sizeof(struct leader_record));
 
+	if (no_delay) {
+		usleep(10000);
+		goto reread;
+	}
+
 	delay = 2 * other_io_timeout;
 	log_space(sp, "delta_acquire delta_short_delay %d", delay);
 
@@ -487,6 +527,7 @@ int delta_lease_acquire(struct task *task,
 		sleep(1);
 	}
 
+reread:
 	error = delta_lease_leader_read(task, sp->sector_size, sp->io_timeout, disk, space_name, host_id, &leader,
 					"delta_acquire_check");
 	if (error < 0) {
@@ -535,7 +576,11 @@ int delta_lease_renew(struct task *task,
 	uint32_t checksum;
 	uint32_t reap_timeout_msec;
 	uint64_t host_id, id_offset, new_ts, now;
-	int rv, iobuf_len, sector_size;
+	int sector_size = sp->sector_size;
+	int align_size = sp->align_size; /* 1,2,4,8 MB depending on sector_size */
+	int iobuf_len = sp->align_size; /* may increase (from 1MB) for hugepage */
+	int read_len = sp->align_size;  /* not increased (from 1MB) for hugepage */
+	int rv;
 
 	if (!leader_last) {
 		log_erros(sp, "delta_renew no leader_last");
@@ -549,15 +594,11 @@ int delta_lease_renew(struct task *task,
 
 	host_id = leader_last->owner_id;
 
-	iobuf_len = sp->align_size;
-
-	sector_size = sp->sector_size;
-
 	/* offset of our leader_record */
 	id_offset = (host_id - 1) * sector_size;
-	if (id_offset > iobuf_len) {
-		log_erros(sp, "delta_renew bad offset %llu iobuf_len %d",
-			  (unsigned long long)id_offset, iobuf_len);
+	if (id_offset > align_size) {
+		log_erros(sp, "delta_renew bad offset %llu align_size %d",
+			  (unsigned long long)id_offset, align_size);
 		return -EINVAL;
 	}
 
@@ -592,7 +633,7 @@ int delta_lease_renew(struct task *task,
 		clock_gettime(CLOCK_MONOTONIC_RAW, &begin);
 
 		rv = read_iobuf_reap(disk->fd, disk->offset,
-				     task->iobuf, iobuf_len, task, reap_timeout_msec);
+				     task->iobuf, read_len, task, reap_timeout_msec);
 
 		log_space(sp, "delta_renew reap %d", rv);
 
@@ -636,23 +677,50 @@ int delta_lease_renew(struct task *task,
 	 */
 
 	if (!task->iobuf) {
+		int read_mem_align = getpagesize();
+		int is_huge = 0;
+
 		/* this will happen the first time renew is called, and after
 		   a timed out renewal read fails to be reaped (see
 		   task->iobuf = NULL above) */
 
 		p_iobuf = &task->iobuf;
 
-		rv = posix_memalign((void *)p_iobuf, getpagesize(), iobuf_len);
+		/*
+		 * Minimum hugepage is 2MB, so for 1MB lease area we increase
+		 * the iobuf size and alignment to 2MB, wasting a MB of space
+		 * to more likely avoid splitting the i/o (read_len remains
+		 * 1MB even though iobuf_len is extended to 2MB.)
+		 * iobuf_len begins being set to align_size (1MB, 2MB, 4MB, 8MB).
+		 */
+		if (com.use_hugepages && ((iobuf_len == ONE_MB_IN_BYTES) || !(iobuf_len % TWO_MB_IN_BYTES))) {
+			read_mem_align = TWO_MB_IN_BYTES;
+			if (iobuf_len < TWO_MB_IN_BYTES)
+				iobuf_len = TWO_MB_IN_BYTES;
+			is_huge = 1;
+		}
+
+		rv = posix_memalign((void *)p_iobuf, read_mem_align, iobuf_len);
 		if (rv) {
 			log_erros(sp, "dela_renew memalign rv %d", rv);
 			rv = -ENOMEM;
 		}
+
+		if (is_huge) {
+			char *iobuf = task->iobuf;
+			char *bp;
+
+			madvise(iobuf, iobuf_len, MADV_HUGEPAGE);
+			/* allocate the pages */
+			for (bp = iobuf; bp < iobuf+iobuf_len; bp += TWO_MB_IN_BYTES)
+				memset(bp, 0, 1);
+		}
 	}
 
 	if (log_renewal_level != -1)
 		log_level(sp->space_id, 0, NULL, log_renewal_level, "delta_renew begin read");
 
-	rv = read_iobuf(disk->fd, disk->offset, task->iobuf, iobuf_len, task, sp->io_timeout, rd_ms);
+	rv = read_iobuf(disk->fd, disk->offset, task->iobuf, read_len, task, sp->io_timeout, rd_ms);
 	if (rv) {
 		/* the next time delta_lease_renew() is called, prev_result
 		   will be this rv.  If this rv is SANLK_AIO_TIMEOUT, we'll
@@ -849,7 +917,7 @@ int delta_lease_init(struct task *task,
 	uint32_t checksum;
 
 	if (!io_timeout)
-		io_timeout = DEFAULT_IO_TIMEOUT;
+		io_timeout = com.io_timeout;
 
 	rv = sizes_from_flags(ls->flags, &sector_size, &align_size, &max_hosts, "LSF");
 	if (rv)
@@ -883,7 +951,7 @@ int delta_lease_init(struct task *task,
 		leader.max_hosts = 1;
 		leader.timestamp = LEASE_FREE;
 		leader.io_timeout = io_timeout;
-		strncpy(leader.space_name, ls->name, NAME_ID_SIZE);
+		memcpy(leader.space_name, ls->name, NAME_ID_SIZE);
 		leader.checksum = 0; /* set below */
 
 		/* make the first record invalid so we can do a single atomic
