@@ -55,7 +55,7 @@ static int direct_read_leader_sizes(struct task *task, struct sync_disk *sd,
 
 	memset(data, 0, datalen);
 
-	rv = read_sectors(sd, 4096, 0, 1, data, datalen, task, DEFAULT_IO_TIMEOUT, "read_sector_size");
+	rv = read_sectors(sd, 4096, 0, 1, data, datalen, task, com.io_timeout, "read_sector_size");
 	if (rv < 0) {
 		free(data);
 		return rv;
@@ -133,6 +133,8 @@ static int do_paxos_action(int action, struct task *task, int io_timeout, struct
 	int disks_len, token_len;
 	int j, rv = 0;
 
+	if (!io_timeout)
+		io_timeout = com.io_timeout;
 	if (!io_timeout)
 		io_timeout = DEFAULT_IO_TIMEOUT;
 
@@ -309,7 +311,7 @@ static int do_delta_action(int action,
 	memset(bitmap, 0, sizeof(bitmap));
 
 	if (!io_timeout)
-		io_timeout = DEFAULT_IO_TIMEOUT;
+		io_timeout = com.io_timeout;
 
 	rv = sizes_from_flags(ls->flags, &sector_size, &align_size, &max_hosts, "LSF");
 	if (rv)
@@ -706,7 +708,7 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 		memset(data, 0, sector_size);
 
 		rv = read_sectors(&sd, sector_size, sector_nr, sector_count, data, datalen,
-				  task, DEFAULT_IO_TIMEOUT, "dump");
+				  task, com.io_timeout, "dump");
 
 		magic_in(data, &magic);
 
@@ -728,8 +730,8 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 				if (!lr->owner_id && !lr->owner_generation)
 					continue;
 
-				strncpy(sname, lr->space_name, NAME_ID_SIZE);
-				strncpy(rname, lr->resource_name, NAME_ID_SIZE);
+				memcpy(sname, lr->space_name, NAME_ID_SIZE);
+				memcpy(rname, lr->resource_name, NAME_ID_SIZE);
 
 				printf("%08llu %36s %48s %010llu %04llu %04llu",
 					(unsigned long long)(start_offset + ((sector_nr + i) * sector_size)),
@@ -752,8 +754,8 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 			leader_record_in(lr_end, &lr_in);
 			lr = &lr_in;
 
-			strncpy(sname, lr->space_name, NAME_ID_SIZE);
-			strncpy(rname, lr->resource_name, NAME_ID_SIZE);
+			memcpy(sname, lr->space_name, NAME_ID_SIZE);
+			memcpy(rname, lr->resource_name, NAME_ID_SIZE);
 
 			printf("%08llu %36s %48s %010llu %04llu %04llu %llu",
 			       (unsigned long long)(start_offset + (sector_nr * sector_size)),
@@ -804,7 +806,7 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 			rindex_header_in(rh_end, &rh_in);
 			rh = &rh_in;
 
-			strncpy(sname, rh->lockspace_name, NAME_ID_SIZE);
+			memcpy(sname, rh->lockspace_name, NAME_ID_SIZE);
 
 			printf("%08llu %36s rindex_header 0x%x %d %u %llu\n",
 			       (unsigned long long)(start_offset + (sector_nr * sector_size)),
@@ -860,7 +862,7 @@ int direct_next_free(struct task *task, char *path)
 	struct leader_record lr;
 	struct sync_disk sd;
 	uint64_t sector_nr;
-	int sector_size, sector_count, datalen, align_size;
+	int sector_size = 0, sector_count, datalen, align_size = 0;
 	int rv;
 
 	memset(&sd, 0, sizeof(struct sync_disk));
@@ -872,7 +874,7 @@ int direct_next_free(struct task *task, char *path)
 		sd.offset = atoll(off_str);
 	}
 
-	strncpy(sd.path, path, SANLK_PATH_LEN);
+	strncpy(sd.path, path, SANLK_PATH_LEN-1);
 	sd.fd = -1;
 
 	rv = open_disk(&sd);
@@ -906,7 +908,7 @@ int direct_next_free(struct task *task, char *path)
 		memset(data, 0, sector_size);
 
 		rv = read_sectors(&sd, sector_size, sector_nr, 1, data, datalen,
-				  task, DEFAULT_IO_TIMEOUT, "next_free");
+				  task, com.io_timeout, "next_free");
 
 		lr_end = (struct leader_record *)data;
 
