@@ -139,16 +139,16 @@ static int write_sysfs_size(const char *disk_path, const char *name, unsigned in
 
 int set_max_sectors_kb(struct sync_disk *disk, uint32_t set_kb)
 {
-	unsigned int max_kb = 0;
+	unsigned int cur_kb = 0;
 	int rv;
 
-	rv = read_sysfs_size(disk->path, "max_sectors_kb", &max_kb);
+	rv = read_sysfs_size(disk->path, "max_sectors_kb", &cur_kb);
 	if (rv < 0) {
 		log_debug("set_max_sectors_kb read error %d %s", rv, disk->path);
 		return rv;
 	}
 
-	if (max_kb == set_kb)
+	if (cur_kb >= set_kb)
 		return 0;
 
 	rv = write_sysfs_size(disk->path, "max_sectors_kb", set_kb);
@@ -914,13 +914,13 @@ int write_sectors(const struct sync_disk *disk, int sector_size, uint64_t sector
 
 /* read aligned io buffer */
 
-int read_iobuf(int fd, uint64_t offset, char *iobuf, int iobuf_len,
+int read_iobuf(int fd, uint64_t offset, char *iobuf, int len,
 	       struct task *task, int ioto, int *rd_ms)
 {
 	if (task && task->use_aio)
-		return do_read_aio_linux(fd, offset, iobuf, iobuf_len, task, ioto, rd_ms);
+		return do_read_aio_linux(fd, offset, iobuf, len, task, ioto, rd_ms);
 	else
-		return do_read(fd, offset, iobuf, iobuf_len, task, rd_ms);
+		return do_read(fd, offset, iobuf, len, task, rd_ms);
 }
 
 /* read sector_count sectors starting with sector_nr, where sector_nr
@@ -977,7 +977,7 @@ int read_sectors(const struct sync_disk *disk, int sector_size, uint64_t sector_
    The aicb used in a task's last timed out read_iobuf is
    task->read_iobuf_timeout_aicb . */
 
-int read_iobuf_reap(int fd, uint64_t offset, char *iobuf, int iobuf_len,
+int read_iobuf_reap(int fd, uint64_t offset, char *iobuf, int read_len,
 		    struct task *task, uint32_t ioto_msec)
 {
 	struct timespec ts;
@@ -995,7 +995,7 @@ int read_iobuf_reap(int fd, uint64_t offset, char *iobuf, int iobuf_len,
 		return -EINVAL;
 	if (iocb->u.c.buf != iobuf)
 		return -EINVAL;
-	if (iocb->u.c.nbytes != iobuf_len)
+	if (iocb->u.c.nbytes != read_len)
 		return -EINVAL;
 	if (iocb->u.c.offset != offset)
 		return -EINVAL;
@@ -1044,9 +1044,9 @@ int read_iobuf_reap(int fd, uint64_t offset, char *iobuf, int iobuf_len,
 			rv = event.res;
 			goto out;
 		}
-		if (event.res != iobuf_len) {
+		if (event.res != read_len) {
 			log_taskw(task, "aio collect %s %p:%p:%p result %ld:%ld match len %d r",
-				  op_str, ev_aicb, ev_iocb, ev_aicb->buf, event.res, event.res2, iobuf_len);
+				  op_str, ev_aicb, ev_iocb, ev_aicb->buf, event.res, event.res2, read_len);
 			rv = -EMSGSIZE;
 			goto out;
 		}
