@@ -54,7 +54,7 @@ static int helper_set_max_sectors_kb(struct sync_disk *disk, uint32_t set_kb)
 	struct helper_msg hm;
 	struct stat st;
 	int ma, mi;
-	unsigned int max_kb = 0;
+	unsigned int cur_kb = 0;
 	int rv;
 
 	rv = stat(disk->path, &st);
@@ -68,13 +68,13 @@ static int helper_set_max_sectors_kb(struct sync_disk *disk, uint32_t set_kb)
 	snprintf(sysfs_path, sizeof(sysfs_path), "/sys/dev/block/%d:%d/queue/max_sectors_kb", ma, mi);
 	sysfs_path[SANLK_HELPER_PATH_LEN-1] = '\0';
 
-	rv = read_sysfs_uint(sysfs_path, &max_kb);
+	rv = read_sysfs_uint(sysfs_path, &cur_kb);
 	if (rv < 0) {
 		/* max_sectors_kb setting may not exist */
 		return -1;
 	}
 
-	if (max_kb == set_kb)
+	if (cur_kb >= set_kb)
 		return 0;
 
 	if (helper_kill_fd == -1)
@@ -82,8 +82,8 @@ static int helper_set_max_sectors_kb(struct sync_disk *disk, uint32_t set_kb)
 
 	memset(&hm, 0, sizeof(hm));
 	hm.type = HELPER_MSG_WRITE_SYSFS;
-	memcpy(hm.path, sysfs_path, SANLK_HELPER_PATH_LEN);
-	snprintf(hm.args, sizeof(hm.args), "%u", set_kb);
+	memcpy(hm.path, sysfs_path, SANLK_HELPER_PATH_LEN-1);
+	snprintf(hm.args, sizeof(hm.args)-1, "%u", set_kb);
 
  retry:
 	rv = write(helper_kill_fd, &hm, sizeof(hm));
@@ -384,6 +384,10 @@ void check_other_leases(struct space *sp, char *buf)
 	struct sanlk_host_event he;
 	char *bitmap;
 	uint64_t now;
+	uint64_t last;
+	uint32_t other_io_timeout;
+	int other_host_fail_seconds;
+	int other_host_dead_seconds;
 	int i, new;
 
 	now = monotime();
@@ -401,6 +405,9 @@ void check_other_leases(struct space *sp, char *buf)
 		leader_record_in(leader_end, &leader_in);
 		leader = &leader_in;
 
+		if (!hs->owner_id && !leader->owner_id)
+			continue;
+
 		/*
 		 * If this lease has invalid fields, log an error.  Limit the logging
 		 * frequency to avoid blowing up logs if some lease is left in a bad
@@ -449,15 +456,102 @@ void check_other_leases(struct space *sp, char *buf)
 				  (unsigned long long)leader->owner_generation,
 				  (unsigned long long)leader->timestamp,
 				  leader->resource_name);
-			strncpy(hs->owner_name, leader->resource_name, NAME_ID_SIZE);
+			memcpy(hs->owner_name, leader->resource_name, NAME_ID_SIZE);
+
+			/* log warning for each generation of a host_id */
+			hs->warned_fail = 0;
+			hs->warned_dead = 0;
 		}
 
+		/*
+		 * Other host's timestamp has not changed since we last checked it.
+		 */
 		if (hs->owner_id == leader->owner_id &&
 		    hs->owner_generation == leader->owner_generation &&
 		    hs->timestamp == leader->timestamp) {
+
+			/*
+			 * No host state to update since the lease is unchanged.
+			 * If host check debugging is enabled, we may want to
+			 * log info about another host lease fail/dead state.
+			 */
+			if (!com.debug_hosts)
+				continue;
+
+			/*
+			 * Our own renewal info is logged elsewhere, this
+			 * is about other hosts lease renewals.
+			 */
+			if (sp->host_id == hs->owner_id)
+				continue;
+
+			/*
+			 * Other host lease is released, not failing or dead.
+			 */
+			if (!hs->timestamp)
+				continue;
+
+			/*
+			 * Don't bother logging info about other host leases if
+			 * we are quitting/failing ourself.
+			 */
+			if (sp->renew_fail || sp->space_dead || sp->killing_pids)
+				continue;
+
+			other_io_timeout = hs->io_timeout;
+			other_host_fail_seconds = calc_id_renewal_fail_seconds(other_io_timeout);
+			other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
+			last = hs->last_live ? hs->last_live : hs->first_check;
+
+			/*
+			 * Informational/debug only, these fail/dead
+			 * determinations are not used anywhere.
+			 */
+
+			if (!hs->warned_fail &&
+			    (com.debug_hosts == 2) &&
+			    (hs->last_check - last > other_host_fail_seconds)) {
+				log_space(sp, "host %llu %llu %llu is failing",
+					  (unsigned long long)hs->owner_id,
+					  (unsigned long long)hs->owner_generation,
+					  (unsigned long long)hs->timestamp);
+				hs->warned_fail = 1;
+			}
+
+			if (!hs->warned_dead &&
+			    (hs->last_check - last > other_host_dead_seconds)) {
+				log_warns(sp, "host %llu %llu %llu is dead",
+					 (unsigned long long)hs->owner_id,
+					 (unsigned long long)hs->owner_generation,
+					 (unsigned long long)hs->timestamp);
+				hs->warned_dead = 1;
+			}
+
 			continue;
 		}
 
+		/*
+		 * Log debug messages for each renewal seen from another host.
+		 */
+		if ((com.debug_hosts == 2) && hs->owner_id) { /* hs->owner_id: skip first time set */
+			if ((hs->owner_id != leader->owner_id) || (hs->owner_generation != leader->owner_generation)) {
+				log_space(sp, "host %llu %llu %llu updated to %llu %llu %llu",
+					  (unsigned long long)hs->owner_id,
+					  (unsigned long long)hs->owner_generation,
+					  (unsigned long long)hs->timestamp,
+					  (unsigned long long)leader->owner_id,
+					  (unsigned long long)leader->owner_generation,
+					  (unsigned long long)leader->timestamp);
+			} else if (hs->timestamp != leader->timestamp) {
+				/* Normal renewal, or release if new timestamp is 0. */
+				log_space(sp, "host %llu %llu %llu new timestamp %llu",
+					  (unsigned long long)hs->owner_id,
+					  (unsigned long long)hs->owner_generation,
+					  (unsigned long long)hs->timestamp,
+					  (unsigned long long)leader->timestamp);
+			}
+		}
+
 		/*
 		 * Replacing good values with potentially bad values
 		 * would have no purpose, and would confuse things, so
@@ -469,7 +563,7 @@ void check_other_leases(struct space *sp, char *buf)
 		if (!hs->lease_bad) {
 			hs->owner_id = leader->owner_id;
 			hs->owner_generation = leader->owner_generation;
-			strncpy(hs->owner_name, leader->resource_name, NAME_ID_SIZE);
+			memcpy(hs->owner_name, leader->resource_name, NAME_ID_SIZE);
 			hs->io_timeout = leader->io_timeout;
 		}
 
@@ -658,14 +752,12 @@ static void save_renewal_history(struct space *sp, int delta_result,
 	}
 }
 
-#define ONE_MB_IN_BYTES 1048576
-#define ONE_MB_IN_KB 1024
-
 static void set_lockspace_max_sectors_kb(struct space *sp, int sector_size, int align_size)
 {
 	struct stat st;
 	int align_size_kb = align_size / 1024; /* align_size is in bytes */
 	unsigned int hw_kb = 0;
+	unsigned int cur_kb = 0;
 	unsigned int set_kb = 0;
 	int rv;
 
@@ -693,6 +785,17 @@ static void set_lockspace_max_sectors_kb(struct space *sp, int sector_size, int
 		return;
 	}
 
+	rv = read_sysfs_size(sp->host_id_disk.path, "max_sectors_kb", &cur_kb);
+	if (rv < 0 || !cur_kb) {
+		log_space(sp, "set_lockspace_max_sectors_kb max_sectors_kb unknown %d %u", rv, cur_kb);
+		return;
+	}
+
+	if (cur_kb >= set_kb) {
+		log_space(sp, "set_lockspace_max_sectors_kb keep current %u for set %u", cur_kb, set_kb);
+		return;
+	}
+
 	if (hw_kb < set_kb) {
 		/*
 		 * If the hardware won't support requested size, try setting 1MB.
@@ -841,6 +944,27 @@ static void *lockspace_thread(void *arg_in)
 		goto set_status;
 	}
 
+	/*
+	 * Tell wdmd to open the watchdog device, set the fire timeout and
+	 * begin the keepalive loop that regularly pets the watchdog.  This
+	 * only happens for the first client/lockspace.  This fails if the
+	 * watchdog device cannot be opened by wdmd or does not support the
+	 * requested fire timeout.
+	 *
+	 * For later clients/lockspaces, when wdmd already has the watchdog
+	 * open, this does nothing (just verifies that fire timeout matches
+	 * what's in use.)
+	 */
+	rv = open_watchdog(wd_con, com.watchdog_fire_timeout);
+	if (rv < 0) {
+		log_erros(sp, "open_watchdog with fire_timeout %d failed %d",
+			  com.watchdog_fire_timeout, wd_con);
+		acquire_result = SANLK_WD_ERROR;
+		delta_result = -1;
+		disconnect_watchdog(sp);
+		goto set_status;
+	}
+
 	/*
 	 * acquire the delta lease
 	 */
@@ -989,7 +1113,7 @@ static void *lockspace_thread(void *arg_in)
 	/* watchdog unlink was done in main_loop when thread_stop was set, to
 	   get it done as quickly as possible in case the wd is about to fire. */
 
-	close_watchdog(sp);
+	disconnect_watchdog(sp);
  out:
 	if (delta_result == SANLK_OK)
 		delta_lease_release(&task, sp, &sp->host_id_disk,
@@ -1036,9 +1160,10 @@ int add_lockspace_start(struct sanlk_lockspace *ls, uint32_t io_timeout, struct
 	int i;
 
 	if (!ls->name[0] || !ls->host_id || !ls->host_id_disk.path[0]) {
-		log_error("add_lockspace bad args id %llu name %zu path %zu",
+		log_error("add_lockspace bad args host_id %llu name %s path %s",
 			  (unsigned long long)ls->host_id,
-			  strlen(ls->name), strlen(ls->host_id_disk.path));
+			  ls->name[0] ? "set" : "empty",
+			  ls->host_id_disk.path[0] ? "set" : "empty");
 		return -EINVAL;
 	}
 
@@ -1306,6 +1431,17 @@ int rem_lockspace_start(struct sanlk_lockspace *ls, unsigned int *space_id)
 		goto out;
 	}
 
+	if (sp->flags & SP_USED_BY_ORPHANS) {
+		/* lock ordering: spaces_mutex, then resource_mutex (resource_orphan_count) */
+		int orphans = resource_orphan_count(sp->space_name);
+		if (orphans) {
+			log_space(sp, "rem_lockspace ignored for orphan count %d", orphans);
+			pthread_mutex_unlock(&spaces_mutex);
+			rv = -EBUSY;
+			goto out;
+		}
+	}
+
 	/*
 	 * Removal happens in a round about way:
 	 * - we set external_remove
@@ -1461,52 +1597,54 @@ int get_lockspaces(char *buf, int *len, int *count, int maxlen)
  * After 80 seconds, we'd return FAIL.  After 140 seconds we'd return DEAD.
  */
 
-/* Also see host_live() */
-
 static uint32_t get_host_flag(struct space *sp, struct host_status *hs)
 {
-	uint64_t now, last;
-	uint32_t flags;
 	uint32_t other_io_timeout;
 	int other_host_fail_seconds, other_host_dead_seconds;
 
-	now = monotime();
+	/* we know we are alive */
+	if (sp->host_id == hs->owner_id)
+		return SANLK_HOST_LIVE;
+
+	/* See sh_holder_live() for same/similar checks */
+
+	if (!hs->first_check)
+		return SANLK_HOST_UNKNOWN;
+
+	if (!hs->last_check)
+		return SANLK_HOST_UNKNOWN;
+
+	if (!hs->timestamp)
+		return SANLK_HOST_FREE;
+
+	if (hs->first_check == hs->last_check)
+		return SANLK_HOST_UNKNOWN;
+
+	if (hs->last_check == hs->last_live)
+		return SANLK_HOST_LIVE;
+
 	other_io_timeout = hs->io_timeout;
 	other_host_fail_seconds = calc_id_renewal_fail_seconds(other_io_timeout);
 	other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
 
-	flags = 0;
-
-	if (!hs->timestamp) {
-		flags = SANLK_HOST_FREE;
-		goto out;
+	if (!hs->last_live) {
+		if (hs->last_check - hs->first_check <= other_host_fail_seconds)
+			return SANLK_HOST_UNKNOWN;
+		if (hs->last_check - hs->first_check > other_host_dead_seconds)
+			return SANLK_HOST_DEAD;
+		if (hs->last_check - hs->first_check > other_host_fail_seconds)
+			return SANLK_HOST_FAIL;
+		/* shouldn't be possible */
+		return SANLK_HOST_UNKNOWN;
+	} else {
+		if (hs->last_check - hs->last_live > other_host_dead_seconds)
+			return SANLK_HOST_DEAD;
+		if (hs->last_check - hs->last_live > other_host_fail_seconds)
+			return SANLK_HOST_FAIL;
+		/* last_check == last_live is the common case, covered above */
+		/* last_check > last_live, but not yet fail_seconds, is live */
+		return SANLK_HOST_LIVE;
 	}
-
-	if (!hs->last_live)
-		last = hs->first_check;
-	else
-		last = hs->last_live;
-
-	if (sp->host_id == hs->owner_id) {
-		/* we are alive */
-		flags = SANLK_HOST_LIVE;
-
-	} else if ((now - last <= other_host_fail_seconds) &&
-		   (hs->first_check == hs->last_live)) {
-		/* we haven't seen the timestamp change yet */
-		flags = SANLK_HOST_UNKNOWN;
-
-	} else if (now - last <= other_host_fail_seconds) {
-		flags = SANLK_HOST_LIVE;
-
-	} else if (now - last > other_host_dead_seconds) {
-		flags = SANLK_HOST_DEAD;
-
-	} else if (now - last > other_host_fail_seconds) {
-		flags = SANLK_HOST_FAIL;
-	}
-out:
-	return flags;
 }
 
 int get_hosts(struct sanlk_lockspace *ls, char *buf, int *len, int *count, int maxlen)
