@@ -20,6 +20,7 @@
 #include <syslog.h>
 #include <sys/types.h>
 #include <sys/time.h>
+#include <sys/mman.h>
 
 #include "sanlock_internal.h"
 #include "diskio.h"
@@ -51,6 +52,36 @@ int get_rand(int a, int b);
 #define BK_DEBUG_COUNT 4
 #define BK_STR_SIZE 80
 
+static void bk_debug_append(char *bk_debug, char *bk_str)
+{
+	int off = 0;
+	int i, j;
+
+	/* bk_debug buffer is BK_DEBUG_SIZE, bk_str is BK_STR_SIZE */
+	/* include as much debugging text in bk_debug as will fit */
+
+	for (i = 0; i < BK_DEBUG_SIZE - 1; i++) {
+		if (!bk_debug[i]) {
+			off = i;
+			break;
+		}
+	}
+
+	if (off >= (BK_DEBUG_SIZE-1)) {
+		bk_debug[BK_DEBUG_SIZE-1] = '\0';
+		return;
+	}
+
+	for (j = 0; j < strlen(bk_str); j++) {
+		bk_debug[off++] = bk_str[j];
+		if (off >= BK_DEBUG_SIZE) {
+			bk_debug[BK_DEBUG_SIZE-1] = '\0';
+			return;
+		}
+	}
+}
+
+/*
 static uint32_t roundup_power_of_two(uint32_t val)
 {
 	val--;
@@ -62,6 +93,7 @@ static uint32_t roundup_power_of_two(uint32_t val)
 	val++;
 	return val;
 }
+*/
 
 uint32_t leader_checksum(struct leader_record *lr)
 {
@@ -452,7 +484,7 @@ static int verify_dblock(struct token *token, struct paxos_dblock *pd, uint32_t
 
 static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 		      int num_hosts, uint64_t next_lver, uint64_t our_mbal,
-		      struct paxos_dblock *dblock_out)
+		      struct paxos_dblock *dblock_out, uint64_t *other_max_mbal, uint64_t *larger_lver)
 {
 	char bk_debug[BK_DEBUG_SIZE];
 	char bk_str[BK_STR_SIZE];
@@ -463,34 +495,81 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 	struct paxos_dblock *bk_end;
 	struct paxos_dblock *bk;
 	struct sync_disk *disk;
-	char *iobuf[SANLK_MAX_DISKS];
+	char *iobuf_array[SANLK_MAX_DISKS];
 	char **p_iobuf[SANLK_MAX_DISKS];
+	char *iobuf;
 	uint32_t checksum;
 	int num_disks = token->r.num_disks;
 	int num_writes, num_reads;
-	int sector_size = token->sector_size;
-	int sector_count;
+	int sector_size;
+	int read_len;
 	int iobuf_len;
+	int read_mem_align;
+	int use_huge = 0;
+	int use_task_hugebuf = 0;
 	int phase2 = 0;
 	int d, q, rv = 0;
 	int q_max = -1;
 	int error;
 
-	sector_count = roundup_power_of_two(num_hosts + 2);
+	sector_size = token->sector_size;
+	/* token->align_size: 1,2,4,8 MB depending on sector_size */
+	read_len = token->align_size;   /* not increased (from 1MB) for hugepage */
+	iobuf_len = token->align_size;  /* can increase (from 1MB) for hugepage */
+	read_mem_align = getpagesize(); /* increases to 2MB for hugepage */
 
-	iobuf_len = sector_count * sector_size;
+	/* Determine if hugepages/hubebuf is applicable. */
+	if (com.use_hugepages && (num_disks == 1) &&
+	    ((iobuf_len == ONE_MB_IN_BYTES) || !(iobuf_len % TWO_MB_IN_BYTES))) {
+		read_mem_align = TWO_MB_IN_BYTES;
+		if (iobuf_len < TWO_MB_IN_BYTES)
+			iobuf_len = TWO_MB_IN_BYTES;
+		use_huge = 1;
+	}
 
-	if (!iobuf_len)
-		return -EINVAL;
+	/* Reuse a previously allocated buffer. */
+	if (use_huge && task->paxos_hugebuf && (task->paxos_hugebuf_size >= iobuf_len)) {
+		use_task_hugebuf = 1;
+		iobuf_array[0] = task->paxos_hugebuf;
+		task->paxos_hugebuf_used++;
+		goto mem_done;
+	}
 
+	/* Allocate a new buffer. */
 	for (d = 0; d < num_disks; d++) {
-		p_iobuf[d] = &iobuf[d];
+		p_iobuf[d] = &iobuf_array[d];
 
-		rv = posix_memalign((void *)p_iobuf[d], getpagesize(), iobuf_len);
+		rv = posix_memalign((void *)p_iobuf[d], read_mem_align, iobuf_len);
 		if (rv)
 			return rv;
 	}
 
+	/* Make the new buffer use hugepages. */
+	if (use_huge) {
+		char *bp;
+		iobuf = iobuf_array[0];
+		madvise(iobuf, iobuf_len, MADV_HUGEPAGE);
+		/* allocate the pages */
+		for (bp = iobuf; bp < iobuf+iobuf_len; bp += TWO_MB_IN_BYTES)
+			memset(bp, 0, 1);
+
+		/*
+		 * Save the new buffer to reuse if the task doesn't
+		 * have one (Note: the task may already have paxos_hugebuf
+		 * with a smaller size than we need here, in which case we
+		 * ignore that, allocate a new one, and free it at the end.)
+		 * TODO: add config min_hugebuf_size to avoid that case?
+		 */
+		if (!task->paxos_hugebuf) {
+			task->paxos_hugebuf = iobuf;
+			task->paxos_hugebuf_size = iobuf_len;
+			task->paxos_hugebuf_created++;
+			use_task_hugebuf = 1;
+			log_taskd(task, "created hugebuf %p %u", iobuf, task->paxos_hugebuf_created);
+		}
+	}
+
+ mem_done:
 
 	/*
 	 * phase 1
@@ -517,7 +596,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 	num_writes = 0;
 
 	for (d = 0; d < num_disks; d++) {
-		/* acquire io: write 1 */
+		/* acquire io: write 1 (sector) */
 		rv = write_dblock(task, token, &token->disks[d], token->host_id, &dblock);
 		if (rv < 0)
 			continue;
@@ -539,20 +618,30 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 	for (d = 0; d < num_disks; d++) {
 		disk = &token->disks[d];
 
-		if (!iobuf[d])
+		if (!iobuf_array[d])
 			continue;
-		memset(iobuf[d], 0, iobuf_len);
 
-		/* acquire io: read 2 */
-		rv = read_iobuf(disk->fd, disk->offset, iobuf[d], iobuf_len, task, token->io_timeout, NULL);
-		if (rv == SANLK_AIO_TIMEOUT)
-			iobuf[d] = NULL;
+		iobuf = iobuf_array[d];
+
+		memset(iobuf, 0, read_len);
+
+		/* acquire io: read 2 (lease area) */
+		rv = read_iobuf(disk->fd, disk->offset, iobuf, read_len, task, token->io_timeout, NULL);
+		if (rv == SANLK_AIO_TIMEOUT) {
+			iobuf_array[d] = NULL;
+			if (use_task_hugebuf) {
+				/* hugebuf will be freed when aio completes */
+				task->paxos_hugebuf = NULL;
+				task->paxos_hugebuf_size = 0;
+				use_task_hugebuf = 0;
+			}
+		}
 		if (rv < 0)
 			continue;
 		num_reads++;
 
 		for (q = 0; q < num_hosts; q++) {
-			bk_end = (struct paxos_dblock *)(iobuf[d] + ((2 + q)*sector_size));
+			bk_end = (struct paxos_dblock *)(iobuf + ((2 + q)*sector_size));
 
 			checksum = dblock_checksum(bk_end);
 
@@ -577,7 +666,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 					 (unsigned long long)bk_in.lver,
 					 bk_in.flags);
 				bk_str[BK_STR_SIZE-1] = '\0';
-				strncat(bk_debug, bk_str, BK_STR_SIZE-1);
+				bk_debug_append(bk_debug, bk_str);
 				bk_debug_count++;
 			}
 
@@ -610,6 +699,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 				log_token(token, "ballot %llu phase1 read %s",
 					  (unsigned long long)next_lver, bk_debug);
 
+				*larger_lver = bk->lver;
 				error = SANLK_DBLOCK_LVER;
 				goto out;
 			}
@@ -636,6 +726,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 				log_token(token, "ballot %llu phase1 read %s",
 					  (unsigned long long)next_lver, bk_debug);
 
+				*other_max_mbal = bk->mbal;
 				error = SANLK_DBLOCK_MBAL;
 				goto out;
 			}
@@ -730,7 +821,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 	num_writes = 0;
 
 	for (d = 0; d < num_disks; d++) {
-		/* acquire io: write 2 */
+		/* acquire io: write 2 (sector) */
 		rv = write_dblock(task, token, &token->disks[d], token->host_id, &dblock);
 		if (rv < 0)
 			continue;
@@ -752,20 +843,30 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 	for (d = 0; d < num_disks; d++) {
 		disk = &token->disks[d];
 
-		if (!iobuf[d])
+		if (!iobuf_array[d])
 			continue;
-		memset(iobuf[d], 0, iobuf_len);
 
-		/* acquire io: read 3 */
-		rv = read_iobuf(disk->fd, disk->offset, iobuf[d], iobuf_len, task, token->io_timeout, NULL);
-		if (rv == SANLK_AIO_TIMEOUT)
-			iobuf[d] = NULL;
+		iobuf = iobuf_array[d];
+
+		memset(iobuf, 0, read_len);
+
+		/* acquire io: read 3 (lease area) */
+		rv = read_iobuf(disk->fd, disk->offset, iobuf, read_len, task, token->io_timeout, NULL);
+		if (rv == SANLK_AIO_TIMEOUT) {
+			iobuf_array[d] = NULL;
+			if (use_task_hugebuf) {
+				/* hugebuf will be freed when aio completes */
+				task->paxos_hugebuf = NULL;
+				task->paxos_hugebuf_size = 0;
+				use_task_hugebuf = 0;
+			}
+		}
 		if (rv < 0)
 			continue;
 		num_reads++;
 
 		for (q = 0; q < num_hosts; q++) {
-			bk_end = (struct paxos_dblock *)(iobuf[d] + ((2 + q)*sector_size));
+			bk_end = (struct paxos_dblock *)(iobuf + ((2 + q)*sector_size));
 
 			checksum = dblock_checksum(bk_end);
 
@@ -790,7 +891,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 					 (unsigned long long)bk->lver,
 					 bk->flags);
 				bk_str[BK_STR_SIZE-1] = '\0';
-				strncat(bk_debug, bk_str, BK_STR_SIZE-1);
+				bk_debug_append(bk_debug, bk_str);
 				bk_debug_count++;
 			}
 
@@ -829,6 +930,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 				log_token(token, "ballot %llu phase2 read %s",
 					  (unsigned long long)next_lver, bk_debug);
 
+				*larger_lver = bk->lver;
 				error = SANLK_DBLOCK_LVER;
 				goto out;
 			}
@@ -855,6 +957,7 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 				log_token(token, "ballot %llu phase2 read %s",
 					  (unsigned long long)next_lver, bk_debug);
 
+				*other_max_mbal = bk->mbal;
 				error = SANLK_DBLOCK_MBAL;
 				goto out;
 			}
@@ -876,11 +979,19 @@ static int run_ballot(struct task *task, struct token *token, uint32_t flags,
 	memcpy(dblock_out, &dblock, sizeof(struct paxos_dblock));
 	error = SANLK_OK;
  out:
-	for (d = 0; d < num_disks; d++) {
-		/* don't free iobufs that have timed out */
-		if (!iobuf[d])
-			continue;
-		free(iobuf[d]);
+	if (use_task_hugebuf) {
+		if (rv != SANLK_AIO_TIMEOUT) {
+			/* Keep buffer to reuse. */
+			memset(task->paxos_hugebuf, 0, task->paxos_hugebuf_size);
+		}
+	} else {
+		for (d = 0; d < num_disks; d++) {
+			/* don't free iobufs that have timed out */
+			if (!iobuf_array[d])
+				continue;
+			free(iobuf_array[d]);
+			iobuf_array[d] = NULL;
+		}
 	}
 
 	if (phase2 && (error < 0) &&
@@ -1343,25 +1454,70 @@ static int _lease_read_one(struct task *task,
 	struct paxos_dblock bk;
 	char *iobuf, **p_iobuf;
 	uint32_t host_id = token->host_id;
-	uint32_t sector_size = token->sector_size;
 	uint32_t checksum;
 	struct paxos_dblock *bk_end;
 	uint64_t tmp_mbal = 0;
-	int q, tmp_q = -1, rv, iobuf_len;
+	int sector_size;
+	int read_len;
+	int iobuf_len;
+	int read_mem_align;
+	int use_huge = 0;
+	int use_task_hugebuf = 0;
+	int q, tmp_q = -1, rv;
 
-	iobuf_len = token->align_size;
-	if (iobuf_len < 0)
-		return iobuf_len;
+	/* FIXME: what case is this? */
+	if (token->align_size < 0)
+		return token->align_size;
 
+	sector_size = token->sector_size;
+	/* token->align_size: 1,2,4,8 MB depending on sector_size */
+	read_len = token->align_size;    /* not increased (from 1MB) for hugepage */
+	iobuf_len = token->align_size;   /* can increase (from 1MB) for hugepage */
+	read_mem_align = getpagesize();  /* increases to 2MB for hugepage */
+
+	/* Determine if hugepages/hubebuf is applicable. */
+	if (com.use_hugepages && ((iobuf_len == ONE_MB_IN_BYTES) || !(iobuf_len % TWO_MB_IN_BYTES))) {
+		read_mem_align = TWO_MB_IN_BYTES;
+		if (iobuf_len < TWO_MB_IN_BYTES)
+			iobuf_len = TWO_MB_IN_BYTES;
+		use_huge = 1;
+	}
+
+	/* Reuse a previously allocated buffer. */
+	if (use_huge && task->paxos_hugebuf && (task->paxos_hugebuf_size >= iobuf_len)) {
+		use_task_hugebuf = 1;
+		iobuf = task->paxos_hugebuf;
+		task->paxos_hugebuf_used++;
+		goto mem_done;
+	}
+
+	/* Allocate a new buffer. */
 	p_iobuf = &iobuf;
-
-	rv = posix_memalign((void *)p_iobuf, getpagesize(), iobuf_len);
+	rv = posix_memalign((void *)p_iobuf, read_mem_align, iobuf_len);
 	if (rv)
 		return rv;
 
-	memset(iobuf, 0, iobuf_len);
+	/* Make the new buffer use hugepages. */
+	if (use_huge) {
+		char *bp;
+		madvise(iobuf, iobuf_len, MADV_HUGEPAGE);
+		/* allocate the pages */
+		for (bp = iobuf; bp < iobuf+iobuf_len; bp += TWO_MB_IN_BYTES)
+			memset(bp, 0, 1);
 
-	rv = read_iobuf(disk->fd, disk->offset, iobuf, iobuf_len, task, token->io_timeout, NULL);
+		/* Save the new buffer to reuse if the task doesn't have one. */
+		if (!task->paxos_hugebuf) {
+			task->paxos_hugebuf = iobuf;
+			task->paxos_hugebuf_size = iobuf_len;
+			task->paxos_hugebuf_created++;
+			use_task_hugebuf = 1;
+			log_taskd(task, "created hugebuf %p %u", iobuf, task->paxos_hugebuf_created);
+		}
+	}
+
+ mem_done:
+
+	rv = read_iobuf(disk->fd, disk->offset, iobuf, read_len, task, token->io_timeout, NULL);
 	if (rv < 0)
 		goto out;
 
@@ -1407,7 +1563,7 @@ static int _lease_read_one(struct task *task,
 				 (unsigned long long)bk.lver,
 				 bk.flags);
 			bk_str[BK_STR_SIZE-1] = '\0';
-			strncat(bk_debug, bk_str, BK_STR_SIZE-1);
+			bk_debug_append(bk_debug, bk_str);
 			bk_debug_count++;
 		}
 
@@ -1432,8 +1588,20 @@ static int _lease_read_one(struct task *task,
 			  bk_debug);
 
  out:
-	if (rv != SANLK_AIO_TIMEOUT)
-		free(iobuf);
+	if (use_task_hugebuf) {
+		if (rv != SANLK_AIO_TIMEOUT) {
+			/* Keep buffer to reuse. */
+			memset(task->paxos_hugebuf, 0, task->paxos_hugebuf_size);
+		} else {
+			/* hugebuf will be freed when aio completes */
+			task->paxos_hugebuf = NULL;
+			task->paxos_hugebuf_size = 0;
+		}
+	} else {
+		if (rv != SANLK_AIO_TIMEOUT)
+			free(iobuf);
+		/* else iobuf will be freed when aio completes */
+	}
 	return rv;
 }
 
@@ -1629,19 +1797,19 @@ static int write_new_leader(struct task *task,
  */
 
 /*
- * i/o required to acquire a free lease
- * (1 disk in token, 512 byte sectors, default num_hosts of 2000)
+ * acquire io: disk ios needed to acquire a free and uncontended lease
  *
  * paxos_lease_acquire()
- * 	paxos_lease_read()	1 read   1 MB (entire lease area)
- * 	run_ballot()
- * 		write_dblock()	1 write  512 bytes (1 dblock sector)
- * 		read_iobuf()	1 read   1 MB (round up num_hosts + 2 sectors)
- * 		write_dblock()  1 write  512 bytes (1 dblock sector)
- * 		read_iobuf()	1 read   1 MB (round up num_hosts + 2 sectors)
- * 	write_new_leader()	1 write  512 bytes (1 leader sector)
+ * R1   paxos_lease_read()	1 read   1 MB (lease area size)
+ *      run_ballot()
+ * W1           write_dblock()	1 write  512 bytes (1 dblock sector)
+ * R2           read_iobuf()	1 read   1 MB (lease area size)
+ * W2           write_dblock()  1 write  512 bytes (1 dblock sector)
+ * R3           read_iobuf()	1 read   1 MB (lease area size)
+ * W3   write_new_leader()	1 write  512 bytes (1 leader sector)
  *
  * 				6 i/os = 3 1MB reads, 3 512 byte writes
+ * 				(assuming 512 sector size and 1MB area size)
  */
 
 int paxos_lease_acquire(struct task *task,
@@ -1663,9 +1831,11 @@ int paxos_lease_acquire(struct task *task,
 	uint64_t wait_start, now;
 	uint64_t last_timestamp;
 	uint64_t next_lver;
+	uint64_t larger_lver = 0;
 	uint64_t max_mbal;
 	uint64_t num_mbal;
 	uint64_t our_mbal;
+	uint64_t other_max_mbal;
 	int copy_cur_leader;
 	int disk_open = 0;
 	int error, rv, us;
@@ -1688,7 +1858,7 @@ int paxos_lease_acquire(struct task *task,
 	memset(&tmp_leader, 0, sizeof(tmp_leader));
 	copy_cur_leader = 0;
 
-	/* acquire io: read 1 */
+	/* acquire io: read 1 (lease area) */
 	error = paxos_lease_read(task, token, flags, &cur_leader, &max_mbal, "paxos_acquire", 1);
 	if (error < 0)
 		goto out;
@@ -1790,6 +1960,7 @@ int paxos_lease_acquire(struct task *task,
 	}
 
 	rv = host_info(cur_leader.space_name, cur_leader.owner_id, &hs);
+
 	if (!rv && hs.last_check && hs.last_live &&
 	    hs.owner_id == cur_leader.owner_id &&
 	    hs.owner_generation == cur_leader.owner_generation) {
@@ -1874,15 +2045,34 @@ int paxos_lease_acquire(struct task *task,
 		 *
 		 * 2. If our own renewal thread saw the owner's timestamp change
 		 * the last time it was checked, then consider the owner to be alive.
+		 *
+		 * first_check != last_live: avoid returning IDLIVE when this
+		 * checks the host state soon after add_lockspace, before more
+		 * than one check_other_leases() has happened.  Before two
+		 * checks of the other lease, a dead host and live host look
+		 * the same (and first_check == last_live).  If the lease owner
+		 * is dead before acquire, then acquire will wait for the lease
+		 * to expire.  If the caller doesn't want acquire to wait for a
+		 * lease to expire, they include the NOWAIT flag.
 		 */
 
 		if ((host_id_leader.timestamp != last_timestamp) ||
-		    (hs.last_live && (hs.last_check == hs.last_live))) {
+		    (hs.last_live && (hs.last_check == hs.last_live) && (hs.first_check != hs.last_live))) {
+
 			log_token(token, "paxos_acquire owner %llu delta %llu %llu %llu alive",
 				  (unsigned long long)cur_leader.owner_id,
 				  (unsigned long long)host_id_leader.owner_id,
 				  (unsigned long long)host_id_leader.owner_generation,
 				  (unsigned long long)host_id_leader.timestamp);
+
+			log_token(token, "paxos_acquire alive host info %llu %llu timestamp %llu first_check %llu last_check %llu last_live %llu",
+				  (unsigned long long)hs.owner_id,
+				  (unsigned long long)hs.owner_generation,
+				  (unsigned long long)hs.timestamp,
+				  (unsigned long long)hs.first_check,
+				  (unsigned long long)hs.last_check,
+				  (unsigned long long)hs.last_live);
+
 			memcpy(leader_ret, &cur_leader, sizeof(struct leader_record));
 
 			/* It's possible that the live owner has released the
@@ -2092,7 +2282,22 @@ int paxos_lease_acquire(struct task *task,
 		goto restart;
 	}
 
-	error = run_ballot(task, token, flags, cur_leader.num_hosts, next_lver, our_mbal, &dblock);
+	other_max_mbal = 0;
+
+	/*
+	 * Another host wrote an lver into their dblock, and then failed before
+	 * that lver could be committed to the leader record.  Advance the next
+	 * lver past the larger lver last seen in a dblock, otherwise ballots
+	 * will continue to be aborted.
+	 */
+	if (larger_lver > next_lver) {
+		log_warnt(token, "paxos_acquire larger lver %llu found beyond next lver %llu",
+			  (unsigned long long)larger_lver, (unsigned long long)next_lver);
+		next_lver = larger_lver + 1;
+	}
+
+	/* acquire io: write 1 (sector), read 2 (lease area), write 2 (sector), read 3 (lease area) */
+	error = run_ballot(task, token, flags, cur_leader.num_hosts, next_lver, our_mbal, &dblock, &other_max_mbal, &larger_lver);
 
 	if ((error == SANLK_DBLOCK_MBAL) || (error == SANLK_DBLOCK_LVER)) {
 		us = get_rand(0, 1000000);
@@ -2103,7 +2308,33 @@ int paxos_lease_acquire(struct task *task,
 			  (unsigned long long)next_lver, us);
 
 		usleep(us);
-		our_mbal += cur_leader.max_hosts;
+
+		/*
+		 * If other hosts have advanced max_mbal far beyond where we're at, then
+		 * skip ahead to the base max_mbal seen in the last ballot, rather than
+		 * adding just adding max_hosts to our_mbal repeatedly until we catch up.
+		 * (Note: run_ballot returns the first larger mbal it finds, and does not
+		 * scan all dblocks for the largest mbal that exists.  So this doesn't
+		 * always skip ahead as far as it could.  All dblocks could be scanned
+		 * to find the largest mbal, but it would be a bigger code change.)
+		 */
+		if (other_max_mbal > (our_mbal + (2 * cur_leader.max_hosts))) {
+			uint64_t other_host_id, other_base_mbal, our_next_mbal;
+			other_host_id = other_max_mbal % cur_leader.max_hosts;
+			other_base_mbal = other_max_mbal - other_host_id;
+			our_next_mbal = other_base_mbal + token->host_id;
+
+			log_warnt(token, "paxos_acquire %llu other_max_mbal %llu advance our_mbal from %llu to %llu",
+				  (unsigned long long)next_lver,
+				  (unsigned long long)other_max_mbal,
+				  (unsigned long long)our_mbal,
+				  (unsigned long long)our_next_mbal);
+
+			our_mbal = our_next_mbal;
+		} else {
+			our_mbal += cur_leader.max_hosts;
+		}
+
 		goto retry_ballot;
 	}
 
@@ -2146,6 +2377,7 @@ int paxos_lease_acquire(struct task *task,
 
 	new_leader.checksum = 0; /* set after leader_record_out */
 
+	/* acquire io: write 3 (sector) */
 	error = write_new_leader(task, token, &new_leader, "paxos_acquire");
 	if (error < 0) {
 		/* See comment in run_ballot about this flag. */
@@ -2467,8 +2699,8 @@ int paxos_lease_init(struct task *task,
 	leader.sector_size = sector_size;
 	leader.num_hosts = num_hosts;
 	leader.max_hosts = max_hosts;
-	strncpy(leader.space_name, token->r.lockspace_name, NAME_ID_SIZE);
-	strncpy(leader.resource_name, token->r.name, NAME_ID_SIZE);
+	memcpy(leader.space_name, token->r.lockspace_name, NAME_ID_SIZE);
+	memcpy(leader.resource_name, token->r.name, NAME_ID_SIZE);
 	leader.checksum = 0; /* set after leader_record_out */
 
 	memset(&rr, 0, sizeof(rr));
@@ -2506,8 +2738,13 @@ int paxos_lease_init(struct task *task,
 		if (rv == SANLK_AIO_TIMEOUT)
 			aio_timeout = 1;
 
-		if (rv < 0)
+		if (rv < 0) {
+			if (!aio_timeout) {
+				free(iobuf);
+				iobuf = NULL;
+			}
 			return rv;
+		}
 	}
 
 	if (!aio_timeout)
