@@ -35,7 +35,7 @@
 #include "helper.h"
 
 /* from cmd.c */
-void send_state_resource(int fd, struct resource *r, const char *list_name, int pid, uint32_t token_id);
+void send_state_resource(int fd, struct resource *r, const char *list_name, int pid, int client_id, uint32_t token_id);
 
 /* from main.c */
 int get_rand(int a, int b);
@@ -71,8 +71,10 @@ static void free_resource(struct resource *r)
 	struct resource *rtmp = NULL;
 	struct resource *rmin = NULL;
 
-	if (r->lvb)
+	if (r->lvb) {
 		free(r->lvb);
+		r->lvb = NULL;
+	}
 
 	if (resources_free_count < FREE_RES_COUNT) {
 		resources_free_count++;
@@ -107,9 +109,9 @@ static struct resource *get_free_resource(struct token *token, int *token_matche
 
 	/* find a previous r that matches token */
 	list_for_each_entry(r, &resources_free, list) {
-		if (strcmp(r->r.lockspace_name, token->r.lockspace_name))
+		if (memcmp(r->r.lockspace_name, token->r.lockspace_name, SANLK_NAME_LEN))
 			continue;
-		if (strcmp(r->r.name, token->r.name))
+		if (memcmp(r->r.name, token->r.name, SANLK_NAME_LEN))
 			continue;
 		if (r->r.num_disks != token->r.num_disks)
 			continue;
@@ -138,19 +140,19 @@ void send_state_resources(int fd)
 	pthread_mutex_lock(&resource_mutex);
 	list_for_each_entry(r, &resources_held, list) {
 		list_for_each_entry(token, &r->tokens, list)
-			send_state_resource(fd, r, "held", token->pid, token->token_id);
+			send_state_resource(fd, r, "held", token->pid, token->client_id, token->token_id);
 	}
 
 	list_for_each_entry(r, &resources_add, list) {
 		list_for_each_entry(token, &r->tokens, list)
-			send_state_resource(fd, r, "add", token->pid, token->token_id);
+			send_state_resource(fd, r, "add", token->pid, token->client_id, token->token_id);
 	}
 
 	list_for_each_entry(r, &resources_rem, list)
-		send_state_resource(fd, r, "rem", r->pid, 0);
+		send_state_resource(fd, r, "rem", r->pid, 0, 0);
 
 	list_for_each_entry(r, &resources_orphan, list)
-		send_state_resource(fd, r, "orphan", r->pid, 0);
+		send_state_resource(fd, r, "orphan", r->pid, 0, 0);
 	pthread_mutex_unlock(&resource_mutex);
 }
 
@@ -326,70 +328,188 @@ int read_resource_owners(struct task *task, struct token *token,
 	return rv;
 }
 
-/* return 1 (is alive) to force a failure if we don't have enough
+/* return 1 (other host is alive) to force a failure if we don't have enough
    knowledge to know it's really not alive.  Later we could have this sit and
    wait (like paxos_lease_acquire) until we have waited long enough or have
    enough knowledge to say it's safely dead (unless of course we find it is
    alive while waiting) */
 
-static int host_live(char *lockspace_name, uint32_t space_id, uint64_t host_id, uint64_t gen)
+/* check_other_leases is called once each time other leases are successfully
+  read, and sets the host_info fields retrieved here. */
+
+static int sh_holder_live(char *lockspace_name, uint32_t space_id, uint64_t host_id, uint64_t mb_gen)
 {
 	struct host_status hs;
-	uint64_t now;
 	int other_io_timeout, other_host_dead_seconds;
 	int rv;
 
+	/*
+	 * host_info() returns what our own renewal thread has observed about
+	 * other host_id leases in the lockspace.  Our renewal thread could be
+	 * delayed renewing our lease, or get io errors reading the lockspace
+	 * leases, so we need to be cautious when making judgements about other
+	 * host liveness based on this data.
+	 *
+	 * hs.first_check
+	 * 	Local time when we first looked at the host_id lease.
+	 * 	Will only be zero for a short period before our first call
+	 * 	to check_other_leases().
+	 * 	While zero, just assume the other host is alive.
+	 *
+	 * hs.last_check
+	 * 	Local time that we last looked at the host_id lease.
+	 * 	Will only be zero for a short period before our first call
+	 * 	to check_other_leases()
+	 * 	While zero, just assume the other host is alive.
+	 *
+	 * hs.last_live
+	 * 	Local time that we last looked at the host_id lease, and
+	 * 	saw an updated timestamp (vs the prev check.)
+	 * 	Will be zero until three steps occur:
+	 * 	1. we renew our lease, and read other lease, read timestamp=X
+	 * 	2. other host updates its lease, writes timestamp=Y
+	 * 	3. we renew our lease, and read other lease, read timestamp=Y
+	 * 	   because X and Y differ, set last_live to local time.
+	 * 	If a new timestamp is never seen, last_live will remain zero,
+	 * 	in which case the difference between first_check and last_check
+	 * 	can be used to determine the host is dead.
+	 *
+	 * The local time difference between now and last_live cannot be used
+	 * to determine that another host is dead, because this time difference
+	 * can become long if we fail to read other leases, i.e. the other
+	 * lease may have a newer timestamp and we just haven't read it.
+	 *
+	 * If we read another host's lease, record the local time we read it
+	 * (time 'A'), then reread that same lease later (time 'B').  If the
+	 * lease is unchanged (no updated timestamp) between the two reads,
+	 * then the time difference between them (B-A) can be compared to
+	 * other_host_dead_seconds to determine if that host is dead.
+	 * It doesn't matter when our own times A and B occur, or if they
+	 * are extended because of our own failure to renew our lease.
+	 */
+
 	rv = host_info(lockspace_name, host_id, &hs);
 	if (rv) {
-		log_sid(space_id, "host_live %llu %llu yes host_info %d",
-			(unsigned long long)host_id, (unsigned long long)gen, rv);
+		log_sid(space_id, "sh_holder %llu mb_gen %llu LIVE - host_info error %d",
+			(unsigned long long)host_id, (unsigned long long)mb_gen, rv);
+		return 1;
+	}
+
+	if (!hs.first_check) {
+		log_sid(space_id, "sh_holder %llu mb_gen %llu LIVE - host_info not yet checked",
+			(unsigned long long)host_id, (unsigned long long)mb_gen);
 		return 1;
 	}
 
 	if (!hs.last_check) {
-		log_sid(space_id, "host_live %llu %llu yes unchecked",
-			(unsigned long long)host_id, (unsigned long long)gen);
+		/* not possible */
+		log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu LIVE - zero last_check should not happen",
+			(unsigned long long)host_id,
+			(unsigned long long)hs.owner_generation,
+			(unsigned long long)hs.timestamp,
+			(unsigned long long)mb_gen);
 		return 1;
 	}
 
 	/* the host_id lease is free, not being used */
 	if (!hs.timestamp) {
-		log_sid(space_id, "host_live %llu %llu no lease free",
-			(unsigned long long)host_id, (unsigned long long)gen);
+		log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu DEAD - lease free",
+			(unsigned long long)host_id,
+			(unsigned long long)hs.owner_generation,
+			(unsigned long long)hs.timestamp,
+			(unsigned long long)mb_gen);
 		return 0;
 	}
 
-	if (hs.owner_generation > gen) {
-		log_sid(space_id, "host_live %llu %llu no old gen %llu",
-			(unsigned long long)host_id, (unsigned long long)gen,
-			(unsigned long long)hs.owner_generation);
+	if (hs.owner_generation > mb_gen) {
+		log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu DEAD - old mb_gen",
+			(unsigned long long)host_id,
+			(unsigned long long)hs.owner_generation,
+			(unsigned long long)hs.timestamp,
+			(unsigned long long)mb_gen);
 		return 0;
 	}
 
-	now = monotime();
+	if (hs.first_check == hs.last_check) {
+		/* not enough checks to know, assume live */
+		log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu LIVE - first_check and last_check %llu",
+			(unsigned long long)host_id,
+			(unsigned long long)hs.owner_generation,
+			(unsigned long long)hs.timestamp,
+			(unsigned long long)mb_gen,
+			(unsigned long long)hs.last_live);
+		return 1;
+	}
+
+	if (hs.last_check == hs.last_live) {
+		log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu LIVE - last_check and last_live %llu",
+			(unsigned long long)host_id,
+			(unsigned long long)hs.owner_generation,
+			(unsigned long long)hs.timestamp,
+			(unsigned long long)mb_gen,
+			(unsigned long long)hs.last_live);
+		return 1;
+	}
 
 	other_io_timeout = hs.io_timeout;
 	other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
 
-	if (!hs.last_live && (now - hs.first_check > other_host_dead_seconds)) {
-		log_sid(space_id, "host_live %llu %llu no first_check %llu",
-			(unsigned long long)host_id, (unsigned long long)gen,
-			(unsigned long long)hs.first_check);
+	/*
+	 * last_live remains 0 until we see the host write a new timestamp,
+	 * which it may never do if it's dead.
+	 */
+	if (!hs.last_live) {
+		if (hs.last_check - hs.first_check > other_host_dead_seconds) {
+			log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu DEAD - last_live 0 last_check %llu first_check %llu",
+				(unsigned long long)host_id,
+				(unsigned long long)hs.owner_generation,
+				(unsigned long long)hs.timestamp,
+				(unsigned long long)mb_gen,
+				(unsigned long long)hs.last_check,
+				(unsigned long long)hs.first_check);
+			return 0;
+		} else {
+			log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu LIVE - last_live 0 last_check %llu first_check %llu",
+				(unsigned long long)host_id,
+				(unsigned long long)hs.owner_generation,
+				(unsigned long long)hs.timestamp,
+				(unsigned long long)mb_gen,
+				(unsigned long long)hs.last_check,
+				(unsigned long long)hs.first_check);
+			return 1;
+		}
+	}
+
+	/*
+	 * last_live is the local time when we last saw a new timestamp.
+	 * last_check is the local time when we last read the timestamp.
+	 * The host is dead if it has not updated its timestamp in
+	 * last_check-last_live.
+	 *
+	 * Note that 'now' could be much later than last_check if we
+	 * have failed to read the lockspace leases (and renew our own
+	 * lease.)  So, now-last_live is not a correct way to measure
+	 * if the other host is dead.
+	 */
+	if (hs.last_check - hs.last_live > other_host_dead_seconds) {
+		log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu DEAD - last_check %llu last_live %llu other_host_dead_seconds %u",
+			(unsigned long long)host_id,
+			(unsigned long long)hs.owner_generation,
+			(unsigned long long)hs.timestamp,
+			(unsigned long long)mb_gen,
+			(unsigned long long)hs.last_check,
+			(unsigned long long)hs.last_live,
+			other_host_dead_seconds);
 		return 0;
 	}
 
-	if (hs.last_live && (now - hs.last_live > other_host_dead_seconds)) {
-		log_sid(space_id, "host_live %llu %llu no last_live %llu",
-			(unsigned long long)host_id, (unsigned long long)gen,
-			(unsigned long long)hs.last_live);
-		return 0;
-	}
-
-	log_sid(space_id, "host_live %llu %llu yes recent first_check %llu last_live %llu",
-		(unsigned long long)host_id, (unsigned long long)gen,
-		(unsigned long long)hs.first_check,
+	log_sid(space_id, "sh_holder %llu %llu %llu mb_gen %llu LIVE - last_check %llu last_live %llu",
+		(unsigned long long)host_id,
+		(unsigned long long)hs.owner_generation,
+		(unsigned long long)hs.timestamp,
+		(unsigned long long)mb_gen,
+		(unsigned long long)hs.last_check,
 		(unsigned long long)hs.last_live);
-
 	return 1;
 }
 
@@ -451,6 +571,37 @@ static int write_host_block(struct task *task, struct token *token,
 	 * values.
 	 */
 	if (pd) {
+		if (!pd->mbal || !pd->bal || !pd->inp) {
+			/*
+			 * In general this shouldn't happen, but this warning
+			 * has been observed in two cases so far:
+			 * a. RETRACT_PAXOS error handling (TODO: fix this by
+			 *    rereading our dblock before writing it back with
+			 *    zero mblock values.)
+			 * b. releasing lease when the last dblock that was
+			 * returned from paxos_lease_acquire has zero bal
+			 * and zero inp, because our last ballot was
+			 * was aborted, and then we detected another host
+			 * committed us as the owner. The other host committed
+			 * us as the owner using a previous dblock that we wrote
+			 * but had aborted. i.e. we did:
+			 * 1. run_ballot write dblock with non-zero mbal,bal,inp
+			 * 2. abort2 the ballot when we see a larger mbal
+			 * 3. run_ballot write dblock with increased non-zero mbal,
+			 *    bal=0, inp=0
+			 * 4. abort1 the ballot when we see a larger mbal
+			 * 5. read leader and see that another host committed
+			 *    the non-zero mbal,bal,inp we wrote in step 1.
+			 * The "pd" we have here is the dblock we wrote in step 3,
+			 * that includes zero bal and inp.
+			 * I don't believe this is a problem.
+			 */
+			log_warnt(token, "write_host_block with zero dblock values %llu %llu %llu",
+				  (unsigned long long)pd->mbal,
+				  (unsigned long long)pd->bal,
+				  (unsigned long long)pd->inp);
+		}
+
 		if (pd->inp && (pd->inp != token->host_id)) {
 			/* This should never happen, sanity check. */
 			log_errot(token, "Ignore bad dblock while writing mblock %llu:%llu:%llu:%llu",
@@ -627,7 +778,7 @@ static int clear_dead_shared(struct task *task, struct token *token,
 			continue;
 		}
 
-		if (host_live(token->r.lockspace_name, token->space_id, host_id, mb.generation)) {
+		if (sh_holder_live(token->r.lockspace_name, token->space_id, host_id, mb.generation)) {
 			log_token(token, "clear_dead_shared host_id %llu gen %llu alive",
 				  (unsigned long long)host_id, (unsigned long long)mb.generation);
 			live++;
@@ -642,10 +793,10 @@ static int clear_dead_shared(struct task *task, struct token *token,
 		}
 
 		/*
-		 * not an error, just useful to have a record of when we clear a shared
-		 * lock that was left by a failed host.
+		 * may be interesting to have a record of when we clear a
+		 * shared lock that was left by a failed host.
 		 */
-		log_errot(token, "cleared shared lease for dead host_id %llu gen %llu",
+		log_warnt(token, "cleared shared lease for dead host %llu %llu",
 			  (unsigned long long)host_id, (unsigned long long)mb.generation);
 	}
 
@@ -676,6 +827,10 @@ static int read_lvb_block(struct task *task, struct token *token)
 
 	rv = read_iobuf(disk->fd, offset, iobuf, iobuf_len, task, token->io_timeout, NULL);
 
+	/* for io timeout, lvb iobuf is freed when the aio completes, not by free_resource */
+	if (rv == SANLK_AIO_TIMEOUT)
+		r->lvb = NULL;
+
 	return rv;
 }
 
@@ -696,6 +851,10 @@ static int write_lvb_block(struct task *task, struct resource *r, struct token *
 
 	rv = write_iobuf(disk->fd, offset, iobuf, iobuf_len, task, token->io_timeout, NULL);
 
+	/* for io timeout, lvb iobuf is freed when the aio completes, not by free_resource */
+	if (rv == SANLK_AIO_TIMEOUT)
+		r->lvb = NULL;
+
 	return rv;
 }
 
@@ -984,6 +1143,12 @@ static int _release_token(struct task *task, struct token *token,
 		goto out;
 	}
 
+	if (nodisk && opened) {
+		log_token(token, "release_token close only");
+		close_disks(token->disks, token->r.num_disks);
+		goto out;
+	}
+
 	if (nodisk)
 		goto out;
 
@@ -1000,18 +1165,20 @@ static int _release_token(struct task *task, struct token *token,
 		  r_flags, (unsigned long long)lver);
 
 	/*
-	 * In all cases we want to (or can) clear both dblock and mblock.
-	 *
-	 * Cases where we want to release ownership of the leader:
-	 * . releasing ex lease !(r_flags & R_SHARED)
-	 * . R_UNDO_SHARED
-	 * . R_ERASE_ALL
+	 * Cases where we want to release ownership of the leader
+	 * (paxos_lease_release or release_disk):
+	 * . normal release of ex lease !(r_flags & R_SHARED)
+	 *   where we should definitely be the leader owner.
+	 * . R_UNDO_SHARED (only if we are actually leader owner)
+	 * . R_ERASE_ALL   (only if we are actually leader owner)
 	 *
 	 * Cases where we don't want to release ownership of the leader:
 	 * . releasing sh lease: (r_flags & R_SHARED)
 	 */
 
 	if (r_flags & R_ERASE_ALL) {
+		/* TODO: reread our dblock, then write it back with zeroed
+		   mblock fields, so that we do not zero our dblock fields. */
 		rv = write_mblock_zero_dblock_release(task, token);
 		if (rv < 0) {
 			log_errot(token, "release_token erase all write_host_block %d", rv);
@@ -1041,6 +1208,8 @@ static int _release_token(struct task *task, struct token *token,
 			  (unsigned long long)lver, rv);
 
 	} else if (r_flags & R_UNDO_SHARED) {
+		/* TODO: reread our dblock, then write it back with zeroed
+		   mblock fields, so that we do not zero our dblock fields. */
 		rv = write_mblock_zero_dblock_release(task, token);
 		if (rv < 0) {
 			log_errot(token, "release_token undo shared write_host_block %d", rv);
@@ -1135,6 +1304,11 @@ static int _release_token(struct task *task, struct token *token,
 	return SANLK_AIO_TIMEOUT;
 }
 
+static int release_token_nodisk_opened(struct task *task, struct token *token)
+{
+	return _release_token(task, token, NULL, 1, 1);
+}
+
 static int release_token_nodisk(struct task *task, struct token *token)
 {
 	return _release_token(task, token, NULL, 0, 1);
@@ -1774,7 +1948,8 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	/* caller only wants to acquire an orphan */
 
-	if (cmd_flags & only_orphan) {
+	if (only_orphan) {
+		log_debug("acquire_token no orphan found for only_orphan");
 		pthread_mutex_unlock(&resource_mutex);
 		return -ENOENT;
 	}
@@ -1799,7 +1974,8 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	if (new_id) {
 		/* save a record of what this id is for later debugging */
-		log_warnt(token, "resource %.48s:%.48s:%.256s:%llu",
+		log_level(token->space_id, token->res_id, NULL, com.names_log_priority,
+			  "resource %.48s:%.48s:%.256s:%llu",
 			  token->r.lockspace_name,
 			  token->r.name,
 			  token->r.disks[0].path,
@@ -1846,24 +2022,26 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 				usleep(us);
 				goto retry;
 			}
-			/* zero r->leader means not owned and release will just close */
-			release_token_opened(task, token);
+			/* We must not write zero dblock values from write_host_block()! */
+			release_token_nodisk_opened(task, token);
+			if (com.quiet_fail)
+				log_token(token, "acquire_token error: %d held retried", rv);
+			else
+				log_errot(token, "acquire_token error: %d held retried", rv);
 			return SANLK_ACQUIRE_SHRETRY;
 		}
 		if (com.quiet_fail)
-			log_token(token, "acquire_token held error %d", rv);
+			log_token(token, "acquire_token error: %d held", rv);
 		else
-			log_errot(token, "acquire_token held error %d", rv);
-		/* zero r->leader means not owned and release will just close */
-		release_token_opened(task, token);
+			log_errot(token, "acquire_token error: %d held", rv);
+		/* We must not write zero dblock values from write_host_block()! */
+		release_token_nodisk_opened(task, token);
 		return rv;
 	}
 
 	if (rv < 0 && !(token->flags & T_RETRACT_PAXOS)) {
-		log_token(token, "acquire_token disk error %d", rv);
-		r->flags &= ~R_SHARED;
-		/* zero r->leader means not owned and release will just close */
-		release_token_opened(task, token);
+		log_token(token, "acquire_token error: %d", rv);
+		release_token_nodisk_opened(task, token);
 		return rv;
 	}
 
@@ -1872,7 +2050,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 		 * We might own the lease, we don't know, so we need to try to
 		 * release on disk to avoid possibly having an orphan lease on disk.
 		 */
-		log_errot(token, "acquire_token disk error %d RETRACT_PAXOS", rv);
+		log_errot(token, "acquire_token error: %d RETRACT_PAXOS", rv);
 		r->flags &= ~R_SHARED;
 		r->flags |= R_ERASE_ALL;
 		memcpy(&r->leader, &leader, sizeof(struct leader_record));
@@ -2160,8 +2338,8 @@ static void do_request(struct token *tt, int pid, uint32_t force_mode)
 	} else if (force_mode == SANLK_REQ_GRACEFUL) {
 		if (killpath[0]) {
 			hm.type = HELPER_MSG_RUNPATH;
-			memcpy(hm.path, killpath, SANLK_HELPER_PATH_LEN);
-			memcpy(hm.args, killargs, SANLK_HELPER_ARGS_LEN);
+			memcpy(hm.path, killpath, SANLK_HELPER_PATH_LEN-1);
+			memcpy(hm.args, killargs, SANLK_HELPER_ARGS_LEN-1);
 		} else {
 			hm.type = HELPER_MSG_KILLPID;
 			hm.pid = pid;
