@@ -64,7 +64,7 @@ static struct token *setup_rindex_token(struct rindex_info *rx,
 	strcpy(token->r.name, "rindex_lease");
 	token->sector_size = sector_size;
 	token->align_size = align_size;
-	token->io_timeout = spi ? spi->io_timeout : DEFAULT_IO_TIMEOUT;
+	token->io_timeout = spi ? spi->io_timeout : com.io_timeout;
 	token->r.num_disks = 1;
 	token->r.flags |= sanlk_res_sector_size_to_flag(sector_size);
 	token->r.flags |= sanlk_res_align_size_to_flag(align_size);
@@ -105,7 +105,7 @@ static struct token *setup_resource_token(struct rindex_info *rx,
 	memcpy(token->r.name, res_name, SANLK_NAME_LEN);
 	token->sector_size = sector_size;
 	token->align_size = align_size;
-	token->io_timeout = spi ? spi->io_timeout : DEFAULT_IO_TIMEOUT;
+	token->io_timeout = spi ? spi->io_timeout : com.io_timeout;
 	token->r.num_disks = 1;
 	token->r.flags |= sanlk_res_sector_size_to_flag(sector_size);
 	token->r.flags |= sanlk_res_align_size_to_flag(align_size);
@@ -268,7 +268,10 @@ static int read_rindex(struct task *task,
 
 	rv = read_iobuf(rx->disk->fd, rx->disk->offset, iobuf, iobuf_len, task, spi->io_timeout, NULL);
 	if (rv < 0) {
-		free(iobuf);
+		if (rv != SANLK_AIO_TIMEOUT) {
+			free(iobuf);
+			iobuf = NULL;
+		}
 		return rv;
 	}
 
@@ -291,7 +294,7 @@ static int read_rindex_header(struct task *task,
 	if (!sector_size)
 		sector_size = 4096;
 	if (!io_timeout) {
-		io_timeout = DEFAULT_IO_TIMEOUT;
+		io_timeout = com.io_timeout;
 		spi->io_timeout = io_timeout;
 	}
 
@@ -437,7 +440,7 @@ int rindex_format(struct task *task, struct sanlk_rindex *ri)
 	rh.sector_size = sector_size;
 	rh.max_resources = max_resources;
 	rh.rx_offset = rx.disk->offset;
-	strncpy(rh.lockspace_name, rx.ri->lockspace_name, NAME_ID_SIZE);
+	memcpy(rh.lockspace_name, rx.ri->lockspace_name, NAME_ID_SIZE);
 
 	memset(&rh_end, 0, sizeof(struct rindex_header));
 	rindex_header_out(&rh, &rh_end);
@@ -447,7 +450,7 @@ int rindex_format(struct task *task, struct sanlk_rindex *ri)
 	if (com.write_init_io_timeout)
 		write_io_timeout = com.write_init_io_timeout;
 	else
-		write_io_timeout = DEFAULT_IO_TIMEOUT;
+		write_io_timeout = com.io_timeout;
 
 	rv = write_iobuf(rx.disk->fd, rx.disk->offset, iobuf, iobuf_len, task, write_io_timeout, NULL);
 	if (rv < 0) {
