@@ -96,8 +96,8 @@ struct sanlk_disk {
 #define SANLK_RES_SECTOR4K	0x00000200
 
 struct sanlk_resource {
-	char lockspace_name[SANLK_NAME_LEN]; /* terminating \0 not required */
-	char name[SANLK_NAME_LEN]; /* terminating \0 not required */
+	char lockspace_name[SANLK_NAME_LEN] __attribute__ ((nonstring)); /* terminating \0 not required */
+	char name[SANLK_NAME_LEN] __attribute__ ((nonstring)); /* terminating \0 not required */
 	uint64_t lver;     /* use with SANLK_RES_LVER */
 	uint64_t data64;   /* per-resource command-specific data */
 	uint32_t data32;   /* per-resource command-specific data */
@@ -105,7 +105,7 @@ struct sanlk_resource {
 	uint32_t flags;    /* SANLK_RES_ */
 	uint32_t num_disks;
 	/* followed by num_disks sanlk_disk structs */
-	struct sanlk_disk disks[0];
+	struct sanlk_disk disks[];
 };
 
 /* make these values match the RES equivalent in case of typos */
@@ -120,12 +120,12 @@ struct sanlk_rindex {
 	uint32_t flags;		/* SANLK_RIF_ */
 	uint32_t max_resources; /* the max res structs that will follow rindex */
 	uint64_t unused;
-	char lockspace_name[SANLK_NAME_LEN]; /* terminating \0 not required */
+	char lockspace_name[SANLK_NAME_LEN] __attribute__ ((nonstring)); /* terminating \0 not required */
 	struct sanlk_disk disk; /* location of rindex */
 };
 
 struct sanlk_rentry {
-	char name[SANLK_NAME_LEN]; /* terminating \0 not required */
+	char name[SANLK_NAME_LEN] __attribute__ ((nonstring)); /* terminating \0 not required */
 	uint64_t offset;
 	uint32_t flags;
 	uint32_t unused;
@@ -135,11 +135,11 @@ struct sanlk_rentry {
    that requires the extra work of segmenting it by resource name) */
 
 struct sanlk_options {
-	char owner_name[SANLK_NAME_LEN]; /* optional user friendly name */
+	char owner_name[SANLK_NAME_LEN] __attribute__ ((nonstring)); /* optional user friendly name */
 	uint32_t flags;
 	uint32_t len; /* unused and ignored */
 	/* followed by len bytes (unused and ignored) */
-	char str[0];
+	char str[];
 };
 
 #define SANLK_LSF_ADD		0x00000001
@@ -154,7 +154,7 @@ struct sanlk_options {
 #define SANLK_LSF_SECTOR4K	0x00000200
 
 struct sanlk_lockspace {
-	char name[SANLK_NAME_LEN];
+	char name[SANLK_NAME_LEN] __attribute__ ((nonstring));
 	uint64_t host_id;
 	uint32_t flags; /* SANLK_LSF_ */
 	struct sanlk_disk host_id_disk;
