@@ -35,6 +35,10 @@
 
 #include <libaio.h>
 
+#define ONE_MB_IN_BYTES 1048576
+#define TWO_MB_IN_BYTES 2097152
+#define ONE_MB_IN_KB 1024
+
 /* default max number of hosts supported */
 
 #define DEFAULT_MAX_HOSTS 2000
@@ -105,6 +109,7 @@ struct token {
 	struct list_head list; /* resource->tokens */
 	struct resource *resource;
 	int pid;
+	int client_id;
 	uint32_t flags;  /* be careful to avoid using this from different threads */
 	uint32_t token_id;
 	uint32_t res_id;
@@ -179,7 +184,9 @@ struct host_status {
 	uint64_t set_bit_time;
 	uint16_t io_timeout;
 	uint16_t lease_bad;
-	char owner_name[NAME_ID_SIZE];
+	uint8_t warned_fail;
+	uint8_t warned_dead;
+	char owner_name[NAME_ID_SIZE] __attribute__ ((nonstring));
 };
 
 struct renewal_history {
@@ -198,7 +205,7 @@ struct renewal_history {
 
 struct space {
 	struct list_head list;
-	char space_name[NAME_ID_SIZE];
+	char space_name[NAME_ID_SIZE] __attribute__ ((nonstring));
 	uint32_t space_id; /* used to refer to this space instance in log messages */
 	uint64_t host_id;
 	uint64_t host_generation;
@@ -269,10 +276,14 @@ struct task {
 
 	unsigned int io_count;       /* stats */
 	unsigned int to_count;       /* stats */
+	unsigned int paxos_hugebuf_used; /* stats */
+	unsigned int paxos_hugebuf_created; /* stats */
 
 	int use_aio;
 	int cb_size;
-	char *iobuf;
+	int paxos_hugebuf_size;
+	char *paxos_hugebuf;         /* paxos_lease_acquire reads of lease area */
+	char *iobuf;                 /* delta_lease_renew reads of lease area */
 	io_context_t aio_ctx;
 	struct aicb *read_iobuf_timeout_aicb;
 	struct aicb *callbacks;
@@ -280,6 +291,17 @@ struct task {
 
 EXTERN struct task main_task;
 
+struct worker_info {
+	unsigned int busy;
+	unsigned int cmd_last;
+	unsigned int work_count;
+	unsigned int io_count;
+	unsigned int to_count;
+	unsigned int paxos_hugebuf_used;
+	unsigned int paxos_hugebuf_created;
+	unsigned int paxos_hugebuf_size;
+};
+
 /* TODO: change used, suspend, need_free, pid_dead to flags */
 
 #define CL_KILLPATH_PID 0x00000001 /* include pid as killpath arg */
@@ -315,7 +337,7 @@ struct client {
 
 EXTERN struct client *client;
 
-#define WATCHDOG_FIRE_TIMEOUT 60
+#define DEFAULT_WATCHDOG_FIRE_TIMEOUT 60
 #define DEFAULT_USE_AIO 1
 #define DEFAULT_IO_TIMEOUT 10
 #define DEFAULT_GRACE_SEC 40
@@ -341,6 +363,7 @@ struct command_line {
 	int action;				/* ACT_ */
 	int debug;
 	int debug_renew;
+	int debug_hosts;
 	int debug_clients;
 	int debug_io_submit;
 	int debug_io_complete;
@@ -352,13 +375,16 @@ struct command_line {
 	int quiet_fail;
 	int wait;
 	int use_watchdog;
+	int watchdog_fire_timeout;
+	int io_timeout;			/* DEFAULT_IO_TIMEOUT or sanlock.conf io_timeout */
+	int kill_grace_seconds;		/* -g */
+	int kill_grace_set;
 	int high_priority;		/* -h */
 	int get_hosts;			/* -h */
 	int names_log_priority;
 	int mlock_level;
 	int max_worker_threads;
 	int aio_arg;
-	int io_timeout_arg;
 	int write_init_io_timeout;
 	int set_bitmap_seconds;
 	int persistent;
@@ -370,11 +396,13 @@ struct command_line {
 	int clear_arg;
 	int sector_size;
 	int align_size;
+	int use_hugepages;
 	char *uname;			/* -U */
 	int uid;				/* -U */
 	char *gname;			/* -G */
 	int gid;				/* -G */
 	int pid;				/* -p */
+	int cid;				/* -C */
 	char sort_arg;
 	uint64_t host_id;			/* -i */
 	uint64_t host_generation;		/* -g */
@@ -388,7 +416,7 @@ struct command_line {
 	int renewal_history_size;
 	int renewal_read_extend_sec_set; /* 1 if renewal_read_extend_sec is configured */
 	uint32_t renewal_read_extend_sec;
-	char our_host_name[SANLK_NAME_LEN+1];
+	char our_host_name_opt[256]; /* max SANLK_NAME_LEN will be used */
 	char *file_path;
 	char *dump_path;
 	int rindex_op;
@@ -401,6 +429,7 @@ struct command_line {
 EXTERN struct command_line com;
 
 uint32_t cmd_str_to_num(const char *str);
+const char *cmd_num_to_str(int cmd);
 uint64_t cmd_num_to_debug_flag(uint32_t cmd);
 int is_cmd_debug(uint32_t cmd);
 void set_cmd_debug(uint32_t cmd);
@@ -452,10 +481,14 @@ enum {
 };
 
 EXTERN int external_shutdown;
+EXTERN int our_host_name_matches_product_uuid;
 EXTERN char our_host_name_global[SANLK_NAME_LEN+1];
+EXTERN char our_product_uuid[SANLK_NAME_LEN+1];
+EXTERN char our_product_uuid_compact[SANLK_NAME_LEN+1]; /* dash chars omitted from uuid */
+
+EXTERN int daemon_status_num_workers; /* replicate thread pool counter for status reporting */
 
 EXTERN int kill_count_max;
-EXTERN int kill_grace_seconds;
 EXTERN int is_helper;
 EXTERN int helper_ci;
 EXTERN int helper_pid;
