@@ -31,6 +31,8 @@
 /* killpath flags */
 #define SANLK_KILLPATH_PID		0x00000001
 
+#define SANLK_FOR_CLIENT_ID		0x00000010
+
 /*
  * acquire flags
  *
@@ -57,6 +59,7 @@
 #define SANLK_ACQUIRE_ORPHAN		0x00000002
 #define SANLK_ACQUIRE_ORPHAN_ONLY	0x00000004
 #define SANLK_ACQUIRE_OWNER_NOWAIT	0x00000008
+/* SANLK_FOR_CLIENT_ID			0x00000010 defined above */
 
 /*
  * release flags
@@ -87,6 +90,7 @@
 #define SANLK_REL_ALL		0x00000001
 #define SANLK_REL_RENAME	0x00000002
 #define SANLK_REL_ORPHAN	0x00000004
+/* SANLK_FOR_CLIENT_ID		0x00000010 defined above */
 
 /*
  * convert flags
@@ -96,6 +100,12 @@
  */
 
 #define SANLK_CONVERT_OWNER_NOWAIT	0x00000008 /* NB: value must match SANLK_ACQUIRE_OWNER_NOWAIT */
+/* SANLK_FOR_CLIENT_ID			0x00000010 defined above */
+
+/*
+ * inquire flags
+ */
+/* SANLK_FOR_CLIENT_ID			0x00000010 defined above */
 
 /*
  * request flags
