@@ -49,6 +49,10 @@ int res_count = DEFAULT_RES_COUNT;
 int pid_count = DEFAULT_PID_COUNT;
 int one_mode = 0;
 int our_hostid;
+int run_sec;
+int count_sec;
+int error_count;
+int error_range = 1;
 int acquire_rv[MAX_RV];
 int release_rv[MAX_RV];
 
@@ -64,6 +68,7 @@ do { \
 	snprintf(error_buf, 4095, "%ld " fmt "\n", time(NULL), ##args); \
 	printf("ERROR: %s\n", error_buf); \
 	syslog(LOG_ERR, "%s", error_buf); \
+	error_count++; \
 } while (0)
 
 
@@ -112,14 +117,47 @@ static void save_rv(int pid, int rv, int acquire)
 		else
 			release_rv[-rv]++;
 	}
+
+	if (error_range == 1) {
+		switch (rv) {
+		case 0:
+		case -EBUSY:
+			/* -16 */
+		case -EEXIST:
+			/* -17 */
+		case -EAGAIN:
+			/* -11 */
+			break;
+		default:
+			log_error("%d ERROR range %d save_rv %d %d", pid, error_range, rv, acquire);
+			break;
+		};
+
+	} else if (error_range == 2) {
+		switch (rv) {
+		case 0:
+		case -EBUSY:
+			/* -16 */
+		case -EEXIST:
+			/* -17 */
+		case -EAGAIN:
+			/* -11 */
+			break;
+		case -243:
+		case -244:
+		case -245:
+			break;
+		default:
+			log_error("%d ERROR range %d save_rv %d %d", pid, error_range, rv, acquire);
+			break;
+		};
+	}
+
 	return;
 
  fail:
 	log_error("%d save_rv %d %d", pid, rv, acquire);
-	while (1) {
-		sleep(10);
-		printf("%lu %d ERROR save_rv %d %d", time(NULL), pid, rv, acquire);
-	}
+	printf("%lu %d ERROR save_rv %d %d", time(NULL), pid, rv, acquire);
 }
 
 static void display_rv(int pid)
@@ -241,11 +279,10 @@ static int check_lock_state(int pid, int result, int count, char *res_state)
 
 	dump_lock_state(pid);
 
-	while (1) {
-		sleep(10);
-		printf("%lu %d ERROR check_lock_state result %d count %d found %d bad %d res_state %s",
-			time(NULL), pid, result, count, found_count, bad_count, res_state);
-	}
+	printf("%lu %d ERROR check_lock_state result %d count %d found %d bad %d res_state %s",
+		time(NULL), pid, result, count, found_count, bad_count, res_state);
+
+	return -1;
 }
 
 #if 0
@@ -508,6 +545,8 @@ int do_rand_child(void)
 	int iter = 1;
 	int pid = getpid();
 
+	error_count = 0;
+
 	srandom(pid);
 
 	memset(lock_state, 0, sizeof(lock_state));
@@ -598,6 +637,52 @@ int do_rand_child(void)
 		}
 		iter++;
 	}
+	display_rv(pid);
+
+	if (error_count) {
+		printf("pid %d done error_count %d\n", pid, error_count);
+		exit(EXIT_FAILURE);
+	}
+
+	printf("pid %d done\n", pid);
+	exit(EXIT_SUCCESS);
+}
+
+int do_all_child(void)
+{
+	int sx, rx, full;
+	int fd, rv;
+	int pid = getpid();
+
+	srandom(pid);
+
+	memset(lock_state, 0, sizeof(lock_state));
+
+	fd = sanlock_register();
+	if (fd < 0) {
+		log_error("%d sanlock_register error %d", pid, fd);
+		exit(-1);
+	}
+
+	while (!prog_stop) {
+		for (sx = 0; sx < ls_count; sx++) {
+			for (rx = 0; rx < res_count; rx++) {
+				rv = acquire_one(pid, fd, sx, rx, EX, &full);
+				if (!rv)
+					lock_state[sx][rx] = EX;
+			}
+
+			inquire_all(pid, fd);
+
+			for (rx = 0; rx < res_count-1; rx++) {
+				rv = release_one(pid, fd, sx, rx);
+				lock_state[sx][rx] = UN;
+			}
+
+			inquire_all(pid, fd);
+		}
+	}
+
 	display_rv(pid);
 	return 0;
 }
@@ -646,6 +731,9 @@ void get_options(int argc, char *argv[])
 		case 'i':
 			our_hostid = atoi(optionarg);
 			break;
+		case 'S':
+			run_sec = atoi(optionarg);
+			break;
 		case 's':
 			ls_count = atoi(optionarg);
 			if (ls_count > MAX_LS_COUNT) {
@@ -670,6 +758,9 @@ void get_options(int argc, char *argv[])
 		case 'm':
 			one_mode = atoi(optionarg);
 			break;
+		case 'e':
+			error_range = atoi(optionarg);
+			break;
 		default:
 			log_error("unknown option: %c", optchar);
 			exit(EXIT_FAILURE);
@@ -732,6 +823,7 @@ int do_rand(int argc, char *argv[])
 	printf("children running\n");
 
 	while (!prog_stop) {
+#if 0
 		/*
 		 * kill and replace a random pid
 		 */
@@ -760,6 +852,7 @@ int do_rand(int argc, char *argv[])
 		} else {
 			children[i] = pid;
 		}
+#endif
 
 #if 0
 		/*
@@ -803,6 +896,89 @@ int do_rand(int argc, char *argv[])
 
 		add_lockspace(lsi);
 #endif
+
+		if (run_sec && (count_sec >= run_sec))
+			break;
+		count_sec++;
+		sleep(1);
+	}
+
+	printf("stopping pids ");
+
+	for (i = 0; i < pid_count; i++)
+		kill(children[i], SIGTERM);
+
+	while (run_count) {
+		status = 0;
+
+		pid = wait(&status);
+		if (pid > 0) {
+			run_count--;
+
+			if (!WIFEXITED(status)) {
+				error_count++;
+				printf("-");
+			} else if (WEXITSTATUS(status)) {
+				error_count++;
+				printf("x");
+			} else {
+				printf(".");
+			}
+		}
+	}
+	printf("\n");
+
+	if (error_count) {
+		printf("child errors %d\n", error_count);
+		exit(EXIT_FAILURE);
+	}
+
+	return 0;
+}
+
+int do_all(int argc, char *argv[])
+{
+	struct sigaction act;
+	int children[MAX_PID_COUNT];
+	int run_count = 0;
+	int i, rv, pid, status;
+
+	if (argc < 5)
+		return -1;
+
+	memset(&act, 0, sizeof(act));
+	act.sa_handler = sigterm_handler;
+	sigaction(SIGTERM, &act, NULL);
+
+	strcpy(lock_disk_base, argv[2]);
+
+	get_options(argc, argv);
+
+	rv = add_lockspaces();
+	if (rv < 0)
+		return rv;
+
+	printf("forking %d pids\n", pid_count);
+
+	for (i = 0; i < pid_count; i++) {
+		pid = fork();
+
+		if (pid < 0) {
+			log_error("fork %d failed %d run_count %d", i, errno, run_count);
+			break;
+		}
+		if (!pid) {
+			do_all_child();
+			exit(-1);
+		}
+		children[i] = pid;
+		run_count++;
+	}
+
+	printf("children running\n");
+
+	while (!prog_stop) {
+		sleep(1);
 	}
 
 	printf("stopping pids");
@@ -819,6 +995,11 @@ int do_rand(int argc, char *argv[])
 	}
 	printf("\n");
 
+	if (error_count) {
+		printf("error_count %d\n", error_count);
+		exit(EXIT_FAILURE);
+	}
+
 	return 0;
 }
 
@@ -903,6 +1084,9 @@ int main(int argc, char *argv[])
 	else if (!strcmp(argv[1], "rand"))
 		rv = do_rand(argc, argv);
 
+	else if (!strcmp(argv[1], "all"))
+		rv = do_all(argc, argv);
+
 	if (!rv)
 		return 0;
 
@@ -917,6 +1101,8 @@ int main(int argc, char *argv[])
 	printf("  -r <num>  number of resources per lockspace\n");
 	printf("  -p <num>  number of processes\n");
 	printf("  -m <num>  use one mode for all locks, 3 = SH, 5 = EX\n");
+	printf("  -S <num>  seconds to run (0 unlimited)\n");
+	printf("  -e <num>  error range expected (1 single node, 2 multi node)\n");
 	printf("  -D        debug output\n");
 	printf("  -V        verbose debug output\n");
 	printf("\n");
