@@ -15,6 +15,7 @@
 #include <stddef.h>
 #include <grp.h>
 #include <fcntl.h>
+#include <libgen.h>
 #include <string.h>
 #include <errno.h>
 #include <limits.h>
@@ -31,6 +32,7 @@
 #include <sys/un.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
+#include <sys/time.h>
 #include <sys/signalfd.h>
 #include <linux/watchdog.h>
 
@@ -55,19 +57,26 @@
 #define DEFAULT_SOCKET_MODE (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP)
 
 #define WDPATH_SIZE 64
+#define WD_ID_SIZE 64
 
-static int test_interval = DEFAULT_TEST_INTERVAL;
+static int standard_test_interval = DEFAULT_TEST_INTERVAL;
+static int test_interval= DEFAULT_TEST_INTERVAL;
 static int fire_timeout = DEFAULT_FIRE_TIMEOUT;
 static int high_priority = DEFAULT_HIGH_PRIORITY;
 static int daemon_quit;
 static int daemon_debug;
+static int try_timeout;
+static int forcefire;
 static int socket_gid;
 static char *socket_gname = (char *)SOCKET_GNAME;
 static time_t last_keepalive;
 static time_t last_closeunclean;
 static char lockfile_path[PATH_MAX];
+static int test_loop_enable;
 static int dev_fd = -1;
 static int shm_fd;
+static int itco; /* watchdog_identity is "iTCO_wdt" */
+static int daemon_setup_done;
 
 static int allow_scripts;
 static int kill_script_sec;
@@ -75,6 +84,7 @@ static const char *scripts_dir = "/etc/wdmd.d";
 static char watchdog_path[WDPATH_SIZE];
 static char option_path[WDPATH_SIZE];
 static char saved_path[WDPATH_SIZE];
+static char watchdog_identity[WD_ID_SIZE];
 
 struct script_status {
 	uint64_t start;
@@ -97,6 +107,7 @@ struct client {
 	int pid;
 	int pid_dead;
 	int refcount;
+	int internal;
 	uint64_t renewal;
 	uint64_t expire;
 	void *workfn;
@@ -110,11 +121,10 @@ static int client_size = 0;
 static struct client *client = NULL;
 static struct pollfd *pollfd = NULL;
 
-
 #define log_debug(fmt, args...) \
 do { \
 	if (daemon_debug) \
-		fprintf(stderr, "%llu " fmt "\n", (unsigned long long)time(NULL), ##args); \
+		fprintf(stderr, "%llu %s " fmt "\n", (unsigned long long)time(NULL), time_str(), ##args); \
 } while (0)
 
 #define log_error(fmt, args...) \
@@ -123,6 +133,12 @@ do { \
 	syslog(LOG_ERR, fmt, ##args); \
 } while (0)
 
+#define log_info(fmt, args...) \
+do { \
+	log_debug(fmt, ##args); \
+	syslog(LOG_INFO, fmt, ##args); \
+} while (0)
+
 #define log_script(i) \
 	log_error("script %.64s last_result %d start %llu run %u fail %u good %u kill %u long %u", \
 		  scripts[i].name, scripts[i].last_result, \
@@ -141,6 +157,19 @@ static uint64_t monotime(void)
 	return ts.tv_sec;
 }
 
+char time_str_buf[128];
+
+static char *time_str(void)
+{
+	struct timeval cur_time;
+	struct tm time_info;
+
+	gettimeofday(&cur_time, NULL);
+	localtime_r(&cur_time.tv_sec, &time_info);
+	strftime(time_str_buf, sizeof(time_str_buf), "%Y-%m-%d %H:%M:%S ", &time_info);
+	return time_str_buf;
+}
+
 /*
  * test clients
  */
@@ -277,12 +306,13 @@ static void dump_debug(int fd)
 	now = monotime();
 
 	memset(line, 0, sizeof(line));
-	snprintf(line, 255, "wdmd %d socket_gid %d high_priority %d now %llu last_keepalive %llu last_closeunclean %llu allow_scripts %d kill_script_sec %d\n",
+	snprintf(line, 255, "wdmd %d socket_gid %d high_priority %d now %llu last_keepalive %llu last_closeunclean %llu allow_scripts %d kill_script_sec %d fire_timeout %d identity \"%s\"\n",
 		 getpid(), socket_gid, high_priority,
 		 (unsigned long long)now,
 		 (unsigned long long)last_keepalive,
 		 (unsigned long long)last_closeunclean,
-		 allow_scripts, kill_script_sec);
+		 allow_scripts, kill_script_sec, fire_timeout,
+		 watchdog_identity);
 
 	line_len = strlen(line);
 	strncat(debug_buf, line, LINE_SIZE);
@@ -334,6 +364,237 @@ static void dump_debug(int fd)
 	send(fd, debug_buf, debug_len, MSG_NOSIGNAL);
 }
 
+static void _init_test_interval(void)
+{
+	if (fire_timeout >= 60) {
+		standard_test_interval = 10;
+		test_interval = 10;
+	} else if (fire_timeout >= 30 && fire_timeout < 60) {
+		standard_test_interval = 5;
+		test_interval = 5;
+	} else if (fire_timeout >= 10 && fire_timeout < 30) {
+		standard_test_interval = 2;
+		test_interval = 2;
+	} else {
+		standard_test_interval = 1;
+		test_interval = 1;
+	}
+}
+
+static int open_dev(void)
+{
+	int fd;
+
+	if (dev_fd != -1) {
+		log_error("watchdog already open fd %d", dev_fd);
+		return -1;
+	}
+
+	fd = open(watchdog_path, O_WRONLY | O_CLOEXEC);
+	if (fd < 0) {
+		log_error("open %s error %d", watchdog_path, errno);
+		return fd;
+	}
+
+	dev_fd = fd;
+	return 0;
+}
+
+static void close_watchdog(void)
+{
+	int rv;
+
+	if (dev_fd == -1) {
+		log_debug("close_watchdog already closed");
+		return;
+	}
+
+	rv = write(dev_fd, "V", 1);
+	if (rv < 0)
+		log_error("%s disarm write error %d", watchdog_path, errno);
+	else if (daemon_setup_done)
+		log_info("%s disarmed", watchdog_path);
+
+	close(dev_fd);
+	dev_fd = -1;
+}
+
+static void close_watchdog_unclean(void)
+{
+	if (dev_fd == -1)
+		return;
+
+	log_error("%s closed unclean", watchdog_path);
+	close(dev_fd);
+	dev_fd = -1;
+
+	last_closeunclean = monotime();
+}
+
+static void pet_watchdog(void)
+{
+	int rv, unused;
+
+	rv = ioctl(dev_fd, WDIOC_KEEPALIVE, &unused);
+
+	last_keepalive = monotime();
+	log_debug("keepalive %d", rv);
+}
+
+static int _open_watchdog_itco(struct wdmd_header *h)
+{
+	int get_timeout_itco, get_timeout_real, set_timeout_itco, set_timeout_real;
+	int rv;
+
+	/* Don't check dev_fd for -1 because dev_fd will be closed
+	   and set to -1 prior to timeout in close_watchdog_unclean().  */
+
+	if (test_loop_enable)
+		return 0;
+
+	if (!h->fire_timeout)
+		return -1;
+
+	rv = open_dev();
+	if (rv < 0)
+		return -1;
+
+	get_timeout_real = 0;
+	get_timeout_itco = 0;
+
+	rv = ioctl(dev_fd, WDIOC_GETTIMEOUT, &get_timeout_itco);
+	if (rv < 0) {
+		log_error("open_watchdog gettimeout error %d", errno);
+		close_watchdog();
+		return -1;
+	}
+
+	get_timeout_real = get_timeout_itco * 2;
+
+	if (get_timeout_real == h->fire_timeout) {
+		/* success, requested value matches the default value */
+		fire_timeout = get_timeout_real;
+		_init_test_interval();
+		log_error("%s open with timeout %d", watchdog_path, get_timeout_real);
+		pet_watchdog();
+		test_loop_enable = 1;
+		return 0;
+	}
+
+	set_timeout_real = h->fire_timeout;
+	set_timeout_itco = set_timeout_real / 2;
+
+	rv = ioctl(dev_fd, WDIOC_SETTIMEOUT, &set_timeout_itco);
+	if (rv < 0) {
+		log_error("open_watchdog settimeout %d error %d", set_timeout_real, errno);
+		close_watchdog();
+		return -1;
+	}
+
+	get_timeout_real = 0;
+	get_timeout_itco = 0;
+
+	rv = ioctl(dev_fd, WDIOC_GETTIMEOUT, &get_timeout_itco);
+	if (rv < 0) {
+		log_error("open_watchdog gettimeout check error %d", errno);
+		close_watchdog();
+		return -1;
+	}
+
+	get_timeout_real = get_timeout_itco * 2;
+
+	if (get_timeout_real == set_timeout_real) {
+		/* success setting a custom timeout */
+		fire_timeout = get_timeout_real;
+		_init_test_interval();
+		log_error("%s open with timeout %d", watchdog_path, get_timeout_real);
+		pet_watchdog();
+		test_loop_enable = 1;
+		return 0;
+	}
+	
+	/* failed to set a custom timeout */
+	log_error("open_watchdog gettimeout value real %d itco %d expect real %d",
+		  get_timeout_real, get_timeout_itco, set_timeout_real);
+	close_watchdog();
+	return -1;
+}
+
+static int _open_watchdog(struct wdmd_header *h)
+{
+	int get_timeout, set_timeout;
+	int rv;
+
+	if (itco)
+		return _open_watchdog_itco(h);
+
+	/* Don't check dev_fd for -1 because dev_fd will be closed
+	   and set to -1 prior to timeout in close_watchdog_unclean().  */
+
+	if (test_loop_enable)
+		return 0;
+
+	if (!h->fire_timeout)
+		return -1;
+
+	rv = open_dev();
+	if (rv < 0)
+		return -1;
+
+	get_timeout = 0;
+
+	rv = ioctl(dev_fd, WDIOC_GETTIMEOUT, &get_timeout);
+	if (rv < 0) {
+		log_error("open_watchdog gettimeout error %d", errno);
+		close_watchdog();
+		return -1;
+	}
+
+	if (get_timeout == h->fire_timeout) {
+		/* success, requested value matches the default value */
+		fire_timeout = get_timeout;
+		_init_test_interval();
+		log_error("%s open with timeout %d", watchdog_path, get_timeout);
+		pet_watchdog();
+		test_loop_enable = 1;
+		return 0;
+	}
+
+	set_timeout = h->fire_timeout;
+
+	rv = ioctl(dev_fd, WDIOC_SETTIMEOUT, &set_timeout);
+	if (rv < 0) {
+		log_error("open_watchdog settimeout %d error %d", set_timeout, errno);
+		close_watchdog();
+		return -1;
+	}
+
+	get_timeout = 0;
+
+	rv = ioctl(dev_fd, WDIOC_GETTIMEOUT, &get_timeout);
+	if (rv < 0) {
+		log_error("open_watchdog gettimeout check error %d", errno);
+		close_watchdog();
+		return -1;
+	}
+
+	if (get_timeout == set_timeout) {
+		/* success setting a custom timeout */
+		fire_timeout = get_timeout;
+		_init_test_interval();
+		log_error("%s open with timeout %d", watchdog_path, get_timeout);
+		pet_watchdog();
+		test_loop_enable = 1;
+		return 0;
+	}
+	
+	/* failed to set a custom timeout */
+	log_error("open_watchdog gettimeout value %d set %d",
+		  get_timeout, set_timeout);
+	close_watchdog();
+	return -1;
+}
+
 static void process_connection(int ci)
 {
 	struct wdmd_header h;
@@ -378,6 +639,17 @@ static void process_connection(int ci)
 		client[ci].refcount = 0;
 		break;
 
+	case CMD_OPEN_WATCHDOG:
+		memcpy(&h_ret, &h, sizeof(h));
+		rv = _open_watchdog(&h);
+		if (rv < 0)
+			h_ret.fire_timeout = 0;
+		else
+			h_ret.fire_timeout = fire_timeout;
+		log_debug("open_watchdog fire_timeout %u result %u", h.fire_timeout, fire_timeout);
+		send(client[ci].fd, &h_ret, sizeof(h_ret), MSG_NOSIGNAL);
+		break;
+
 	case CMD_TEST_LIVE:
 		client[ci].renewal = h.renewal_time;
 		client[ci].expire = h.expire_time;
@@ -484,6 +756,7 @@ static int setup_clients(void)
 
 	ci = client_add(fd, process_listener, client_pid_dead);
 	strncpy(client[ci].name, "listen", WDMD_NAME_SIZE);
+	client[ci].internal = 1;
 	return 0;
 }
 
@@ -509,7 +782,7 @@ static int test_clients(void)
 
 		if (t >= client[i].expire) {
 			log_error("test failed rem %d now %llu ping %llu close %llu renewal %llu expire %llu client %d %s",
-				  DEFAULT_FIRE_TIMEOUT - (int)(t - last_ping),
+				  fire_timeout - (int)(t - last_ping),
 				  (unsigned long long)t,
 				  (unsigned long long)last_keepalive,
 				  (unsigned long long)last_closeunclean,
@@ -540,7 +813,7 @@ static int test_clients(void)
 		 * expiration time.
 		 */
 
-		if (t >= client[i].expire - DEFAULT_TEST_INTERVAL) {
+		if (t >= client[i].expire - standard_test_interval) {
 			log_error("test warning now %llu ping %llu close %llu renewal %llu expire %llu client %d %s",
 				  (unsigned long long)t,
 				  (unsigned long long)last_keepalive,
@@ -567,6 +840,25 @@ static int active_clients(void)
 	return 0;
 }
 
+static void count_clients(int *active, int *external)
+{
+	int act = 0, ext = 0;
+	int i;
+
+	for (i = 0; i < client_size; i++) {
+		if (!client[i].used)
+			continue;
+		if (client[i].refcount)
+			act++;
+		if (!client[i].internal)
+			ext++;
+	}
+	if (active)
+		*active = act;
+	if (external)
+		*external = ext;
+}
+
 
 #ifdef TEST_FILES
 #define FILES_DIR "/run/wdmd/test_files"
@@ -791,7 +1083,7 @@ static int test_scripts(void)
 		 */
 
 		if (!scripts[i].last_result &&
-		    ((begin - scripts[i].start) < (DEFAULT_TEST_INTERVAL - 1)))
+		    ((begin - scripts[i].start) < (standard_test_interval - 1)))
 			continue;
 
 		pid = run_script(i);
@@ -806,7 +1098,7 @@ static int test_scripts(void)
 		}
 	}
 
-	/* wait up to DEFAULT_TEST_INTERVAL-1 for the pids to finish */
+	/* wait up to standard_test_interval-1 for the pids to finish */
 
 	while (1) {
 		running = 0;
@@ -893,7 +1185,7 @@ static int test_scripts(void)
 		if (!running)
 			break;
 
-		if (monotime() - begin >= DEFAULT_TEST_INTERVAL - 1)
+		if (monotime() - begin >= standard_test_interval - 1)
 			break;
 
 		sleep(1);
@@ -927,58 +1219,49 @@ static int test_scripts(void)
 	return fail_count;
 }
 
-static int open_dev(void)
+static int setup_identity(char *wdpath)
 {
-	int fd;
+	char sysfs_path[PATH_MAX] = { 0 };
+	char *base, *p;
+	int fd, rv;
 
-	if (dev_fd != -1) {
-		log_error("watchdog already open fd %d", dev_fd);
+	/*
+	 * This function will be called multiple times when probing
+	 * different watchdog paths for one that works.
+	 */
+	itco = 0;
+	memset(watchdog_identity, 0, sizeof(watchdog_identity));
+
+	/*
+	 * $ cat /sys/class/watchdog/watchdog0/identity
+	 * iTCO_wdt
+	 */
+	if (!(base = basename(wdpath)))
 		return -1;
-	}
 
-	fd = open(watchdog_path, O_WRONLY | O_CLOEXEC);
-	if (fd < 0) {
-		log_error("open %s error %d", watchdog_path, errno);
-		return fd;
-	}
+	snprintf(sysfs_path, PATH_MAX-1, "/sys/class/watchdog/%s/identity", base);
+
+	if ((fd = open(sysfs_path, O_RDONLY)) < 0)
+		return -1;
+
+	rv = read(fd, watchdog_identity, WD_ID_SIZE-1);
+
+	close(fd);
+
+	if (rv <= 0)
+		return -1;
+
+	if ((p = strchr(watchdog_identity, '\n')))
+		*p = '\0';
+
+	log_debug("%s %s %s", wdpath, sysfs_path, watchdog_identity);
+
+	if (!strcmp(watchdog_identity, "iTCO_wdt"))
+		itco = 1;
 
-	dev_fd = fd;
 	return 0;
 }
 
-static void close_watchdog_unclean(void)
-{
-	if (dev_fd == -1) {
-		log_debug("close_watchdog_unclean already closed");
-		return;
-	}
-
-	log_error("%s closed unclean", watchdog_path);
-	close(dev_fd);
-	dev_fd = -1;
-
-	last_closeunclean = monotime();
-}
-
-static void close_watchdog(void)
-{
-	int rv;
-
-	if (dev_fd == -1) {
-		log_error("close_watchdog already closed");
-		return;
-	}
-
-	rv = write(dev_fd, "V", 1);
-	if (rv < 0)
-		log_error("%s disarm write error %d", watchdog_path, errno);
-	else
-		log_error("%s disarmed", watchdog_path);
-
-	close(dev_fd);
-	dev_fd = -1;
-}
-
 static int _setup_watchdog(char *path)
 {
 	struct stat buf;
@@ -1004,25 +1287,9 @@ static int _setup_watchdog(char *path)
 		return -1;
 	}
 
-	if (timeout == fire_timeout)
-		goto out;
+	log_debug("%s gettimeout reported %u", watchdog_path, timeout);
 
-	timeout = fire_timeout;
-
-	rv = ioctl(dev_fd, WDIOC_SETTIMEOUT, &timeout);
-	if (rv < 0) {
-		log_error("%s failed to set timeout", watchdog_path);
-		close_watchdog();
-		return -1;
-	}
-
-	if (timeout != fire_timeout) {
-		log_error("%s failed to set new timeout", watchdog_path);
-		close_watchdog();
-		return -1;
-	}
- out:
-	log_error("%s armed with fire_timeout %d", watchdog_path, fire_timeout);
+	close_watchdog();
 
 	/* TODO: save watchdog_path in /run/wdmd/saved_path,
 	 * and in startup read that file, copying it to saved_path */
@@ -1031,6 +1298,7 @@ static int _setup_watchdog(char *path)
 }
 
 /*
+ * Success: returns 0 with watchdog_path set.
  * Order of preference:
  * . saved path (path used before daemon restart)
  * . command line option (-w)
@@ -1091,7 +1359,303 @@ static int setup_watchdog(void)
 
 }
 
-static int probe_dev(const char *path)
+/*
+ * iTCO_wdt actual firing timeout is double the value used in get/set!
+ * https://bugzilla.kernel.org/show_bug.cgi?id=213809
+ */
+static int _try_timeout_itco(const char *path)
+{
+	struct stat buf;
+	int try_timeout_real, try_timeout_itco, get_timeout_real, get_timeout_itco, set_timeout_real, set_timeout_itco;
+	int unused, fd, err, rv, rv2;
+
+	rv = stat(path, &buf);
+	if (rv < 0) {
+		fprintf(stderr, "%s stat error %d\n", path, errno);
+		return -1;
+	}
+
+	fd = open(path, O_WRONLY | O_CLOEXEC);
+	if (fd < 0) {
+		fprintf(stderr, "%s open error %d\n", path, errno);
+		return fd;
+	}
+
+	printf("%s %s open fd %d\n", time_str(), path, fd);
+
+	get_timeout_real = 0;
+	get_timeout_itco = 0;
+
+	rv = ioctl(fd, WDIOC_GETTIMEOUT, &get_timeout_itco);
+	if (rv < 0) {
+		fprintf(stderr, "%s gettimeout error %d\n", path, errno);
+		rv = -1;
+		goto out;
+	}
+
+	get_timeout_real = get_timeout_itco * 2;
+
+	printf("%s %s gettimeout real %d itco %d\n", time_str(), path, get_timeout_real, get_timeout_itco);
+
+	if (get_timeout_real == try_timeout)
+		goto keepalive;
+
+	try_timeout_real = try_timeout;
+	try_timeout_itco = try_timeout_real / 2;
+	set_timeout_real = try_timeout;
+	set_timeout_itco = set_timeout_real / 2;
+
+	rv = ioctl(fd, WDIOC_SETTIMEOUT, &set_timeout_itco);
+	if (rv < 0) {
+		fprintf(stderr, "%s settimeout real %d itco %d error %d\n", path, set_timeout_real, set_timeout_itco, errno);
+		rv = -1;
+		goto out;
+	}
+
+	set_timeout_real = set_timeout_itco * 2;
+
+	printf("%s %s settimeout real %d itco %d result real %d itco %d\n", time_str(), path,
+	       try_timeout_real, try_timeout_itco, set_timeout_real, set_timeout_itco);
+
+	if (set_timeout_itco != try_timeout_itco) {
+		fprintf(stderr, "%s settimeout real %d itco %d failed\n", path, try_timeout_real, try_timeout_itco);
+		rv = -1;
+		goto out;
+	}
+
+	get_timeout_real = 0;
+	get_timeout_itco = 0;
+
+	rv = ioctl(fd, WDIOC_GETTIMEOUT, &get_timeout_itco);
+	if (rv < 0) {
+		fprintf(stderr, "%s gettimeout error %d\n", path, errno);
+		rv = -1;
+		goto out;
+	}
+
+	get_timeout_real = get_timeout_itco * 2;
+
+	printf("%s %s gettimeout real %d itco %d\n", time_str(), path, get_timeout_real, get_timeout_itco);
+
+ keepalive:
+
+	rv = ioctl(fd, WDIOC_KEEPALIVE, &unused);
+	if (rv < 0) {
+		fprintf(stderr, "%s keepalive error %d\n", path, errno);
+		rv = -1;
+		goto out;
+	}
+
+	printf("%s %s keepalive fd %d result %d\n", time_str(), path, fd, rv);
+
+	if (forcefire) {
+		int sleep_sec = 0;
+		int i;
+		setbuf(stdout, NULL);
+		printf("%s waiting for watchdog to reset machine:\n", time_str());
+		for (i = 1; i < get_timeout_real + 5; i++) {
+			sleep(1);
+			sleep_sec++;
+			if (sleep_sec >= get_timeout_real+1)
+				printf("%s %d %s failed to fire after timeout %d seconds\n", time_str(), i, path, get_timeout_real);
+			else
+				printf("%s %d\n", time_str(), i);
+		}
+	}
+
+	rv = 0;
+ out:
+	err = write(fd, "V", 1);
+	if (err < 0) {
+		fprintf(stderr, "trytimeout failed to disarm %s error %d %d\n", path, err, errno);
+		openlog("wdmd", LOG_CONS | LOG_PID, LOG_DAEMON);
+		syslog(LOG_ERR, "trytimeout failed to disarm %s error %d %d\n", path, err, errno);
+	}
+
+	printf("%s %s disarm write V fd %d result %d\n", time_str(), path, fd, rv);
+
+	rv2 = close(fd);
+
+	printf("%s %s close fd %d result %d\n", time_str(), path, fd, rv2);
+
+	return rv;
+}
+
+static int _try_timeout(const char *path)
+{
+	struct stat buf;
+	int get_timeout, set_timeout;
+	int unused, fd, err, rv, rv2;
+
+	rv = stat(path, &buf);
+	if (rv < 0) {
+		fprintf(stderr, "%s stat error %d\n", path, errno);
+		return -1;
+	}
+
+	fd = open(path, O_WRONLY | O_CLOEXEC);
+	if (fd < 0) {
+		fprintf(stderr, "%s open error %d\n", path, errno);
+		return fd;
+	}
+
+	printf("%s %s open fd %d\n", time_str(), path, fd);
+
+	get_timeout = 0;
+
+	rv = ioctl(fd, WDIOC_GETTIMEOUT, &get_timeout);
+	if (rv < 0) {
+		fprintf(stderr, "%s gettimeout error %d\n", path, errno);
+		rv = -1;
+		goto out;
+	}
+
+	printf("%s %s gettimeout %d\n", time_str(), path, get_timeout);
+
+	if (get_timeout == try_timeout)
+		goto keepalive;
+
+	set_timeout = try_timeout;
+
+	rv = ioctl(fd, WDIOC_SETTIMEOUT, &set_timeout);
+	if (rv < 0) {
+		fprintf(stderr, "%s settimeout %d error %d\n", path, set_timeout, errno);
+		rv = -1;
+		goto out;
+	}
+
+	printf("%s %s settimeout %d result %d\n", time_str(), path, try_timeout, set_timeout);
+
+	if (set_timeout != try_timeout) {
+		fprintf(stderr, "%s settimeout %d failed\n", path, try_timeout);
+		rv = -1;
+		goto out;
+	}
+
+	get_timeout = 0;
+
+	rv = ioctl(fd, WDIOC_GETTIMEOUT, &get_timeout);
+	if (rv < 0) {
+		fprintf(stderr, "%s gettimeout error %d\n", path, errno);
+		rv = -1;
+		goto out;
+	}
+
+	printf("%s %s gettimeout %d\n", time_str(), path, get_timeout);
+
+ keepalive:
+
+	rv = ioctl(fd, WDIOC_KEEPALIVE, &unused);
+	if (rv < 0) {
+		fprintf(stderr, "%s keepalive error %d\n", path, errno);
+		rv = -1;
+		goto out;
+	}
+
+	printf("%s %s keepalive fd %d result %d\n", time_str(), path, fd, rv);
+
+	if (forcefire) {
+		int sleep_sec = 0;
+		int i;
+		setbuf(stdout, NULL);
+		printf("%s waiting for watchdog to reset machine:\n", time_str());
+		for (i = 1; i < get_timeout + 5; i++) {
+			sleep(1);
+			sleep_sec++;
+			if (sleep_sec >= get_timeout+1)
+				printf("%s %d %s failed to fire after timeout %d seconds\n", time_str(), i, path, get_timeout);
+			else
+				printf("%s %d\n", time_str(), i);
+		}
+	}
+
+	rv = 0;
+ out:
+	err = write(fd, "V", 1);
+	if (err < 0) {
+		fprintf(stderr, "trytimeout failed to disarm %s error %d %d\n", path, err, errno);
+		openlog("wdmd", LOG_CONS | LOG_PID, LOG_DAEMON);
+		syslog(LOG_ERR, "trytimeout failed to disarm %s error %d %d\n", path, err, errno);
+	}
+
+	printf("%s %s disarm write V fd %d result %d\n", time_str(), path, fd, rv);
+
+	rv2 = close(fd);
+
+	printf("%s %s close fd %d result %d\n", time_str(), path, fd, rv2);
+
+	return rv;
+}
+
+static int _probe_dev_itco(const char *path)
+{
+	struct stat buf;
+	int fd, err, rv, timeout_real, timeout_itco;
+
+	rv = stat(path, &buf);
+	if (rv < 0) {
+		fprintf(stderr, "error %d stat %s\n", errno, path);
+		return -1;
+	}
+
+	fd = open(path, O_WRONLY | O_CLOEXEC);
+	if (fd < 0) {
+		fprintf(stderr, "error %d open %s\n", errno, path);
+		return fd;
+	}
+
+	timeout_real = 0;
+	timeout_itco = 0;
+
+	rv = ioctl(fd, WDIOC_GETTIMEOUT, &timeout_itco);
+	if (rv < 0) {
+		fprintf(stderr, "error %d ioctl gettimeout %s\n", errno, path);
+		rv = -1;
+		goto out;
+	}
+
+	timeout_real = timeout_itco * 2;
+
+	if (timeout_real == fire_timeout) {
+		printf("%s\n", path);
+		rv = 0;
+		goto out;
+	}
+
+	timeout_real = fire_timeout;
+	timeout_itco = timeout_real / 2;
+
+	rv = ioctl(fd, WDIOC_SETTIMEOUT, &timeout_itco);
+	if (rv < 0) {
+		fprintf(stderr, "error %d ioctl settimeout %s\n", errno, path);
+		rv = -1;
+		goto out;
+	}
+
+	timeout_real = timeout_itco * 2;
+
+	if (timeout_real != fire_timeout) {
+		fprintf(stderr, "error %d invalid timeout %s\n", errno, path);
+		rv = -1;
+		goto out;
+	}
+
+	printf("%s\n", path);
+	rv = 0;
+
+ out:
+	err = write(fd, "V", 1);
+	if (err < 0) {
+		fprintf(stderr, "probe failed to disarm %s error %d %d\n", path, err, errno);
+		openlog("wdmd", LOG_CONS | LOG_PID, LOG_DAEMON);
+		syslog(LOG_ERR, "probe failed to disarm %s error %d %d\n", path, err, errno);
+	}
+
+	close(fd);
+	return rv;
+}
+
+static int _probe_dev(const char *path)
 {
 	struct stat buf;
 	int fd, err, rv, timeout;
@@ -1153,6 +1717,37 @@ static int probe_dev(const char *path)
 	return rv;
 }
 
+static int probe_dev(const char *wdpath)
+{
+	char *path = (char *)wdpath;
+
+	setup_identity(path);  /* sets itco=1 if iTCO_wdt */
+
+	if (try_timeout) {
+		/*
+		 * Used to test support for a given timeout with: wdmd -t <secs>
+		 * or to test firing for a given timeout with: wdmd -F -t <secs>
+		 */
+		if (itco)
+			return _try_timeout_itco(path);
+		else
+			return _try_timeout(path);
+	} else {
+		/*
+		 * Used to print on stdout just the path of the watchdog device
+		 * that wdmd would use with: wdmd -p
+		 */
+		if (itco)
+			return _probe_dev_itco(path);
+		else
+			return _probe_dev(path);
+	}
+}
+
+/*
+ * Confusingly, this is the top level function for both
+ * wdmd -t (test timeout) and wdmd -p (print functional watchdog device).
+ */
 static int probe_watchdog(void)
 {
 	int rv;
@@ -1205,16 +1800,6 @@ static int probe_watchdog(void)
 
 }
 
-static void pet_watchdog(void)
-{
-	int rv, unused;
-
-	rv = ioctl(dev_fd, WDIOC_KEEPALIVE, &unused);
-
-	last_keepalive = monotime();
-	log_debug("keepalive %d", rv);
-}
-
 static void process_signals(int ci)
 {
 	struct signalfd_siginfo fdsi;
@@ -1257,6 +1842,7 @@ static int setup_signals(void)
 
 	ci = client_add(fd, process_signals, client_pid_dead);
 	strncpy(client[ci].name, "signal", WDMD_NAME_SIZE);
+	client[ci].internal = 1;
 	return 0;
 }
 
@@ -1300,13 +1886,13 @@ static int test_loop(void)
 	void (*workfn) (int ci);
 	void (*deadfn) (int ci);
 	uint64_t test_time;
+	int resetting = 0;
+	int active_usage, external_usage;
 	int poll_timeout;
 	int sleep_seconds;
 	int fail_count;
 	int rv, i;
 
-	pet_watchdog();
-
 	test_time = 0;
 	poll_timeout = test_interval * 1000;
 
@@ -1332,9 +1918,42 @@ static int test_loop(void)
 			}
 		}
 
-		if (daemon_quit && !active_clients())
+		count_clients(&active_usage, &external_usage);
+
+		if (daemon_quit && !active_usage)
 			break;
 
+		/*
+		 * No client has called open_watchdog() so the wd device is not open yet.
+		 */
+		if (!test_loop_enable)
+			continue;
+
+		/*
+		 * active_usage are client connections with a refcount.
+		 * external_usage are any clients other than internal.
+		 * (open_watchdog happens with external but not active
+		 * connections.)
+		 *
+		 * checking resetting here is critical to avoiding
+		 * unnecessary resets: while in recovery mode we
+		 * have done close_watchdog_unclean, then all clients
+		 * are cleared, and we need the loop below to see
+		 * no further failures and reopen and pet the watchdog
+		 * again to avoid a reset.  After it's been reopened,
+		 * and no longer used due to all clients being cleared,
+		 * then it's ok to get here and close cleanly.
+		 */
+		if (!active_usage && !external_usage && !resetting) {
+			log_debug("close watchdog unused");
+			close_watchdog();
+			test_loop_enable = 0;
+			test_interval = standard_test_interval;
+			poll_timeout = test_interval * 1000;
+			test_time = 0;
+			continue;
+		}
+
 		if (monotime() - test_time >= test_interval) {
 			test_time = monotime();
 			log_debug("test_time %llu",
@@ -1354,7 +1973,8 @@ static int test_loop(void)
 					pet_watchdog();
 				}
 
-				test_interval = DEFAULT_TEST_INTERVAL;
+				test_interval = standard_test_interval;
+				resetting = 0;
 			} else {
 				/* If we can patch the kernel so that close
 				   does not generate a ping, then we can skip
@@ -1363,6 +1983,7 @@ static int test_loop(void)
 				close_watchdog_unclean();
 
 				test_interval = RECOVER_TEST_INTERVAL;
+				resetting = 1;
 			}
 		}
 
@@ -1407,8 +2028,7 @@ static int lockfile(void)
 
 	rv = fcntl(fd, F_SETLK, &lock);
 	if (rv < 0) {
-		log_error("lockfile setlk error %s: %s",
-			  lockfile_path, strerror(errno));
+		log_error("daemon already running (lockfile %s %d)", lockfile_path, errno);
 		goto fail;
 	}
 
@@ -1445,19 +2065,19 @@ static void setup_priority(void)
 
 	rv = mlockall(MCL_CURRENT | MCL_FUTURE);
 	if (rv < 0) {
-		log_error("mlockall failed");
+		log_info("mlockall failed");
 	}
 
 	rv = sched_get_priority_max(SCHED_RR);
 	if (rv < 0) {
-		log_error("could not get max scheduler priority err %d", errno);
+		log_info("could not get max scheduler priority err %d", errno);
 		return;
 	}
 
 	sched_param.sched_priority = rv;
 	rv = sched_setscheduler(0, SCHED_RR|SCHED_RESET_ON_FORK, &sched_param);
 	if (rv < 0) {
-		log_error("could not set RR|RESET_ON_FORK priority %d err %d",
+		log_info("could not set RR|RESET_ON_FORK priority %d err %d",
 			  sched_param.sched_priority, errno);
 	}
 }
@@ -1468,7 +2088,7 @@ static int group_to_gid(char *arg)
 
 	gr = getgrnam(arg);
 	if (gr == NULL) {
-		log_error("group '%s' not found, using socket gid: %i", arg, DEFAULT_SOCKET_GID);
+		log_info("group '%s' not found, using socket gid: %i", arg, DEFAULT_SOCKET_GID);
 		return DEFAULT_SOCKET_GID;
 	}
 
@@ -1513,19 +2133,21 @@ static void print_usage_and_exit(int status)
 {
 	printf("Usage:\n");
 	printf("wdmd [options]\n\n");
-	printf("--version, -V         print version\n");
-	printf("--help, -h            print usage\n");
-	printf("--dump, -d            print debug from daemon\n");
-	printf("--probe, -p           print path of functional watchdog device\n");
-	printf("-D                    debug: no fork and print all logging to stderr\n");
-	printf("-H 0|1                use high priority features (1 yes, 0 no, default %d)\n",
-				      DEFAULT_HIGH_PRIORITY);
-	printf("-G <name>             group ownership for the socket\n");
-	printf("-S 0|1                allow script tests (default %d)\n", allow_scripts);
-	printf("-s <path>             path to scripts dir (default %s)\n", scripts_dir);
-	printf("-k <num>              kill unfinished scripts after num seconds (default %d)\n",
-				      kill_script_sec);
-	printf("-w /dev/watchdog      path to the watchdog device to try first\n");
+	printf("--version, -V          print version\n");
+	printf("--help, -h             print usage\n");
+	printf("--dump, -d             print debug from daemon\n");
+	printf("--probe, -p            print path of functional watchdog device\n");
+	printf("--trytimeout, -t <sec> set the timeout value for watchdog device\n");
+	printf("--forcefire, -F        force watchdog to fire and reset machine, use with -t\n");
+	printf("-D                     debug: no fork and print all logging to stderr\n");
+	printf("-H 0|1                 use high priority features (1 yes, 0 no, default %d)\n",
+				       DEFAULT_HIGH_PRIORITY);
+	printf("-G <name>              group ownership for the socket\n");
+	printf("-S 0|1                 allow script tests (default %d)\n", allow_scripts);
+	printf("-s <path>              path to scripts dir (default %s)\n", scripts_dir);
+	printf("-k <num>               kill unfinished scripts after num seconds (default %d)\n",
+				       kill_script_sec);
+	printf("-w <path>              path to the watchdog device to try first\n");
 	exit(status);
 }
 
@@ -1553,14 +2175,16 @@ int main(int argc, char *argv[])
 	    int option_index = 0;
 
 	    static struct option long_options[] = {
-	        {"help",    no_argument, 0,  'h' },
-	        {"probe",   no_argument, 0,  'p' },
-	        {"dump",    no_argument, 0,  'd' },
-	        {"version", no_argument, 0,  'V' },
-	        {0,         0,           0,  0 }
+	        {"help",       no_argument,       0,  'h' },
+	        {"probe",      no_argument,       0,  'p' },
+	        {"dump",       no_argument,       0,  'd' },
+	        {"trytimeout", required_argument, 0,  't' },
+	        {"forcefire",  no_argument,       0,  'F' },
+	        {"version",    no_argument,       0,  'V' },
+	        {0,            0,                 0,  0 }
 	    };
 
-	    c = getopt_long(argc, argv, "hpdVDH:G:S:s:k:w:",
+	    c = getopt_long(argc, argv, "hpdVDH:G:S:s:k:w:t:F",
 	                    long_options, &option_index);
 	    if (c == -1)
 	         break;
@@ -1572,6 +2196,13 @@ int main(int argc, char *argv[])
 		case 'p':
 		    do_probe = 1;
 		    break;
+		case 't':
+		    do_probe = 1;
+		    try_timeout = atoi(optarg);
+		    break;
+		case 'F':
+		    forcefire = 1;
+		    break;
 		case 'd':
 		    print_debug_and_exit();
 		    break;
@@ -1603,6 +2234,11 @@ int main(int argc, char *argv[])
 	    }
 	}
 
+	if (forcefire && !do_probe) {
+		fprintf(stderr, "Use force fire (-F) with a timeout (-t).\n");
+		exit(EXIT_FAILURE);
+	}
+
 	if (do_probe) {
 		rv = setup_shm();
 		if (rv < 0) {
@@ -1633,9 +2269,6 @@ int main(int argc, char *argv[])
 
 	openlog("wdmd", LOG_CONS | LOG_PID, LOG_DAEMON);
 
-	log_error("wdmd started S%d H%d G%d", allow_scripts, high_priority,
-		  socket_gid);
-
 	setup_priority();
 
 	rv = lockfile();
@@ -1662,10 +2295,19 @@ int main(int argc, char *argv[])
 	if (rv < 0)
 		goto out_files;
 
+	/* Sets watchdog_path */
 	rv = setup_watchdog();
 	if (rv < 0)
 		goto out_clients;
 
+	/* Sets watchdog_identity and itco */
+	setup_identity(watchdog_path);
+
+	log_info("wdmd started S%d H%d G%d using %s \"%s\"", allow_scripts, high_priority,
+		 socket_gid, watchdog_path, watchdog_identity[0] ? watchdog_identity : "unknown");
+
+	daemon_setup_done = 1;
+
 	rv = test_loop();
 
 	close_watchdog();
